<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Login extends \Zeedhi\Framework\Controller\Simple {

    protected $loginService;
    protected $snackMessage;
    protected $util;
    protected $instanceManager;
    protected $SMTPService;
    protected $sessionHandler;
    protected $notificationService;

    public function __construct(\Service\Login $loginService, \Util\SnackMessage $snackMessage, \Util\Util $util, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Service\SMTP $SMTPService, \Util\SessionHandler $sessionHandler, \Service\Notification $notificationService){
        $this->loginService = $loginService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
        $this->instanceManager = $instanceManager;
        $this->SMTPService = $SMTPService;
        $this->sessionHandler = $sessionHandler;
        $this->notificationService = $notificationService;
    }

    public function getClients(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        try {
            if(count($params) > 0){
                $clients = $this->loginService->getClientsWithFilter($params[0]['value']);
            } else{
                $clients = $this->loginService->getClients();
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ClientsRepository', $clients));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function passwordConversion(){
        $passwords = $this->loginService->getPasswordConversion();
        foreach ($passwords as $currentPassword) {
            $oldPassword = $this->util->decrypt($currentPassword['CDSENHACONSUMIDORWEB']);
            $newPassword = $this->util->encrypt($oldPassword);
            $this->loginService->updatePasswordConversion($currentPassword['CDCLIENTE'], $currentPassword['CDCONSUMIDOR'], $newPassword);
        }
    }

    public function login(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDCLIENTE      = $params[0]['value'];
            $NMLOGINCONS    = $params[1]['value'];
            $CDSENHACONSMD5 = $params[2]['value'];
            $CDFILIAL       = $params[3]['value'];
            if ($NMLOGINCONS == 'renovando@senha.com'){
                $this->passwordConversion();
            }

            self::checkLoginData($CDCLIENTE, $NMLOGINCONS, $CDSENHACONSMD5);

            $consumer = $this->loginService->getConsumerByLogin($CDCLIENTE, $NMLOGINCONS);
            if (empty($consumer)){
                $consumer = $this->loginService->getConsumerByEmail($CDCLIENTE, $NMLOGINCONS);
                if(empty($consumer)){
                    throw new \Exception($this->snackMessage->getMessage('6')); // Consumidor nÃ£o encontrado.
                }
            }
            if (empty($consumer['CDSENHACONSMD5'])){
                $response->addMessage(new Message($this->snackMessage->getMessage('15'))); // Senha não cadastrada.
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array('nothing' => 'newPassword')));
            } else if (!$this::isPasswordOk($CDSENHACONSMD5, $consumer['CDSENHACONSMD5'])){
                throw new \Exception($this->snackMessage->getMessage('6')); // Senha incorreta.
            } else {
                // Se login for válido, busca os filhotes associados ao CPF do registro encontrado.
                $consumers = $this->loginService->getConsumerByParent($CDCLIENTE, $consumer['NRCPFRESPCON'], $CDFILIAL);
                // Colocando GRIDVALUES no consumer, usando um ponteiro.
                $consumers = $this->getGridValues($consumers);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumersRepository', $consumers));

                // Busca valores para a visão financeira.
                $values = $this->loginService->getFinancialVisionValues($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
                // Retorna junto com os dados de login.
                $consumer['GRIDVALUES'] = $values;
                // Colocando filial.
                $consumer['CDFILIAL'] = $CDFILIAL;

                $consumerId = $CDFILIAL.$consumer['CDCLIENTE'].$consumer['CDCONSUMIDOR'];
                //$this->sessionHandler->setConsumerId($consumerId);

                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LoginRepository', $consumer));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumerRepository', $consumer));
            }
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function reloadConsumers(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDCLIENTE    = $params[0]['value'];
            $NRCPFRESPCON = $params[1]['value'];
            $lang         = $params[2]['value'];
            $CDFILIAL     = $params[3]['value'];

            $consumers = $this->loginService->getConsumerByParent($CDCLIENTE, $NRCPFRESPCON, $CDFILIAL);

            $NRNOTIFNAOLIDA = 0;
            foreach($consumers as $consumer){
                $consumerNonReadedNotifications = $this->notificationService->getNonReadedNotificationsByConsumer($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
                $NRNOTIFNAOLIDA = $NRNOTIFNAOLIDA + (int)$consumerNonReadedNotifications[0]['NRNOTIFNAOLIDA'];
            }

            if($NRNOTIFNAOLIDA === 0){
                $NRNOTIFNAOLIDA = '';
            }

            $consumers = $this->getGridValues($consumers);

            if($lang == 'en_us'){
                $consumers[0]['GRIDVALUES'][0]['NAME'] = 'Initial Balance';
                $consumers[0]['GRIDVALUES'][1]['NAME'] = 'Credit';
                $consumers[0]['GRIDVALUES'][2]['NAME'] = 'Debit';
                $consumers[0]['GRIDVALUES'][3]['NAME'] = 'Current Balance';
            }else if($lang == 'es'){
                $consumers[0]['GRIDVALUES'][0]['NAME'] = 'Equilibrio inicial';
                $consumers[0]['GRIDVALUES'][1]['NAME'] = 'Crédito';
                $consumers[0]['GRIDVALUES'][2]['NAME'] = 'Débito';
                $consumers[0]['GRIDVALUES'][3]['NAME'] = 'Equilibio actual';
            }

            $consumers[0]['NRNOTIFNAOLIDA'] = $NRNOTIFNAOLIDA;

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumersRepository', $consumers));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getGridValues($consumers){
       foreach ($consumers as &$consumer) {
           $values = $this->loginService->getFinancialVisionValues($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
           $consumer['GRIDVALUES'] = $values;
       }
    return $consumers;
    }

    public function setPassword(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE   = $params[0]['value'];
            $NMLOGINCONS = $params[1]['value'];
            $newPassword = $params[2]['value'];
            $CDFILIAL    = $params[3]['value'];
            // change the password
            $CDSENHACONSMD5 = $this->util->encrypt($newPassword);
            $consumer = $this->loginService->getConsumerByLogin($CDCLIENTE, $NMLOGINCONS, false);
            $this->loginService->changePassword($consumer['CDCLIENTE'], $consumer['NMLOGINCONS'], $CDSENHACONSMD5);

            // do login
            $consumerId = $CDFILIAL.$consumer['CDCLIENTE'].$consumer['CDCONSUMIDOR'];
            //$this->sessionHandler->setConsumerId($consumerId);
            $consumer['CDFILIAL'] = $CDFILIAL;
            unset($consumer[0]['CDSENHACONSMD5']);

            // Se login for válido, busca os filhotes associados ao CPF do registro encontrado.
            $consumers = $this->loginService->getConsumerByParent($consumer['CDCLIENTE'], $consumer['NRCPFRESPCON'], $consumer['CDFILIAL']);
            // Colocando GRIDVALUES no consumer, usando um ponteiro.
            $consumers = $this->getGridValues($consumers);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumersRepository', $consumers));

            $response->addMessage(new Message($this->snackMessage->getMessage('16')));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LoginRepository', $consumer));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumerRepository', $consumer));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function isPasswordOk($informedPassword, $dbPassword){
        if ($this->util->encrypt($informedPassword) === $dbPassword){
            return true;
        }
        else {
            return false;
        }
    }

    public function checkLoginData($CDCLIENTE, $NMLOGINCONS, $CDSENHACONSMD5){
        if ($CDCLIENTE == null){
            throw new \Exception($this->snackMessage->getMessage('3'));
        } else if ($NMLOGINCONS == null){
            throw new \Exception($this->snackMessage->getMessage('4'));
        } else if ($CDSENHACONSMD5 == null){
            throw new \Exception($this->snackMessage->getMessage('5'));
        }
    }

    public function forgotPassword(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDCLIENTE   = $params[0]['value'];
            $NMLOGINCONS = $params[1]['value'];
            $baseUrl     = $params[2]['value'];
            $logoEmailInfo  = array(
                'appUrl'    => $baseUrl,
                'logoEmail' => $params[3]['value']
            );

            $NMLOGINCONS = strtolower($NMLOGINCONS);

            if ($CDCLIENTE != null){
                $consumerData = $this->loginService->getConsumerByLogin($CDCLIENTE, $NMLOGINCONS);
                $consumerGroup = null;
            }
            else {
                $consumerGroup = $this->loginService->getConsumerByLoginClientless($NMLOGINCONS);
                if (!empty($consumerGroup) && sizeof($consumerGroup) == 1) $consumerData = $consumerGroup[0];
                else $consumerData = null;
            }

            if ($consumerGroup !== null && sizeof($consumerGroup) > 1){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ClientsRepository', $consumerGroup));
            }
            else if (empty($consumerData)){
                $response->addMessage(new Message($this->snackMessage->getMessage(20)));
            }
            else {
                $CDCLIENTE    = $consumerData['CDCLIENTE'];
                $CDCONSUMIDOR = $consumerData['CDCONSUMIDOR'];
                $NMRESPCONS   = $consumerData['NMRESPCONS'];
                $DSEMAILCONS  = $consumerData['DSEMAILCONS'];
                if (empty($DSEMAILCONS)) throw new \Exception("Consumidor sem e-mail cadastrado. Favor entrar em contato com o administrador do site.");

                $hashConsumer = $this->buildHashCode($CDCONSUMIDOR);
                $hashCliente  = $this->buildHashCode($CDCLIENTE);

                $consumerSize = str_pad(strlen($CDCONSUMIDOR), 2, '0', STR_PAD_LEFT);
                $clientSize   = str_pad(strlen($CDCLIENTE), 2, '0', STR_PAD_LEFT);

                $hash = $hashConsumer.$hashCliente.$consumerSize.$clientSize;
                $linkForChange = $this->generateLink($baseUrl, str_replace(' ', '~', $hash)); // Replaces spaces by squiggly to avoid breaking the link.

                $subject  = $this->snackMessage->getMessage('13');
                $sender = $this->snackMessage->getMessage('14');
                $content =
                    '<p>Olá '.$NMRESPCONS.'!</p>'.
                    '<p>Acesse o link a seguir para alterar sua senha: <b>'.
                    '<a href='.$linkForChange.'>'.'Clique Aqui'.'</a>'.'</b> </p>'
                ;

                $this->SMTPService->sendEmail($NMRESPCONS, $DSEMAILCONS, $sender, $subject, $content, $logoEmailInfo);
                $this->loginService->updateHashAlteraSenha($CDCONSUMIDOR, $CDCLIENTE, $hash);

                $response->addMessage(new Message($this->snackMessage->getMessage(21)));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function updatePassword(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDSENHACONSMD5 = $params[0]['value'];
            $hash = $params[1]['value'];

            $hash = str_replace('~', ' ', $hash); // Replaces squiggly with spaces to get the actual values of the hash.

            $consumerSize = substr($hash, 130, 2);
            $clientSize = substr($hash, 132);

            $CDCONSUMIDOR = $this->getFirstHashHalf($hash, $consumerSize);
            $CDCLIENTE    = $this->getSecondHashHalf($hash, $clientSize);
            $verifyHash   = $this->loginService->getHash($CDCONSUMIDOR, $CDCLIENTE);

            if ($hash == $verifyHash){
                $CDSENHACONSMD5 = $this->util->encrypt($CDSENHACONSMD5);
                $this->loginService->updatePassword($CDCLIENTE, $CDCONSUMIDOR, $CDSENHACONSMD5);
                $this->loginService->updateHashAlteraSenha($CDCONSUMIDOR, $CDCLIENTE, null);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('UpdatePasswordRepo', array('mensagem' => 'Senha alterada com sucesso.')));
            }
            else {
                $response->addMessage(new Message($this->snackMessage->getMessage(22)));
            }

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function buildHashCode($string) {
        $time = microtime(true);

        $hash = $this->util->encrypt($string.$time.rand(0, 99999999));
        $hash2 = $this->util->encrypt(rand(0,335689));
        $hash = $hash . $hash2;
        $hashArray = str_split($hash);
        $operatorArray = str_split($string);

        $parImpar = rand(0, 1);

        $finallyHash = "";
        foreach ($hashArray as $key => $hashValue) {
            if ($key % 2 == $parImpar && count($operatorArray) > 0) {
                $finallyHash .= array_shift($operatorArray);
            } else {
                $finallyHash .= $hashValue;
            }
        }
        $finallyHash .= $parImpar;

        return $finallyHash;
    }

    private function getFirstHashHalf($hash, $length) {
        $hashArray = str_split($hash);
        $hashConsumidor = array_chunk($hashArray, 65, true);
        $hashArray = $hashConsumidor[0];
        $parImpar = $hashArray[count($hashArray) -1];

        $result = "";
        foreach ($hashArray as $key => $hashValue) {
            if ($key % 2 == $parImpar) {
                $result .= $hashValue;
            }
        }

        $result = substr($result, 0, $length);
        return $result;
    }

    private function getSecondHashHalf($hash, $length) {
        $hashArray = str_split($hash);
        $hashCliente = array_chunk($hashArray, 65,true);
        $hashArray = $this->util->indexArray($hashCliente[1], 65, 130); // Reindexa o array.
        $parImpar = $hashArray[count($hashArray) - 1];

        $result = "";
        foreach ($hashArray as $key => $hashValue) {
            if ($key % 2 == $parImpar) {
                $result .= $hashValue;
            }
        }

        $result = substr($result, 0, $length);
        return $result;
    }

    private function generateLink ($baseUrl, $hash) {
        //return $baseUrl . 'mobile?' . $hash; //DEVELOPMENT
        return $baseUrl . '?' . $hash; //PRODUCTION
    }

    public function logoff(Request\Filter $request, Response $response){
        //$this->sessionHandler->endSession();
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LogoffRepository', array()));
    }

    public function getLayoutConfig(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();

        $CDCLIENTE   = $params[0]['value'];
        $CDTIPOCONS    = $params[1]['value'];
        $CDFILIAL    = $params[2]['value'];

        $layoutConfig = $this->loginService->getLayoutConfig($CDCLIENTE, $CDTIPOCONS, $CDFILIAL);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LayoutRepo', $layoutConfig));
    }

}
