<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Exception;

class Product extends \Zeedhi\Framework\Controller\Simple {

    protected $priceController;
    protected $productService;
    protected $snackMessage;
    protected $util;

    public function __construct(\Controller\Price $priceController, \Service\ProductService $productService, \Util\SnackMessage $snackMessage, \Util\Util $util){
        $this->priceController = $priceController;
        $this->productService = $productService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
    }

    public function validateProducts($products, $CDCLIENTE, $CDFILIAL, $CDLOJA){
        try {
            $assembledProducts = array();
            foreach ($products as $product){
                /* Checks if the product exists by getting its details. */
                $productDetails = $this->checkProduct($product['CDPRODUTO'], $CDFILIAL);
                /* Gets product prices. */
                $productPrice = $this->priceController->getPrice($product['CDPRODUTO'], $CDCLIENTE, $CDFILIAL, $CDLOJA);
                /* Combines the price details. */
                $productDetails = array_merge($productDetails, $productPrice);
                /* Process observations. */
                $productDetails['TXPRODCOMVEN'] = $this->processObservations($product);
                /* Sets quantity. */
                $productDetails['QTDPRODUTO'] = $product['qtty'];
                /* Combines all the product's details into one array and adds them to the final array. */
                $productDetails = array_merge($productDetails, $product);
                array_push($assembledProducts, $productDetails);
            }

            return $assembledProducts;
        } catch (\Exception $e) {
            throw new Exception\ValidationException('Erro de validação. \n'.$e->getMessage());
        }
    }

    public function checkProduct($CDPRODUTO, $CDFILIAL){
        try {
            /* Gets the product details. */
            $productDetails = $this->productService->getProductDetails($CDPRODUTO);
            if (Empty($productDetails['CDARVPROD'])){
                throw new Exception\InvalidProductException('Produto não cadastrado.');
            }
            /* Checks if the product is taxed. */
            $aliquote = $this->productService->getProductAliquote($CDFILIAL, $productDetails['CDPRODUTO']);
            if ($aliquote['COUNT'] === 0){
                throw new Exception\UntaxedProductException('Produto sem imposto.');
            }
            return $productDetails;
        } catch (Exception\SqlException $e) {
            throw new Exception\SqlException($this->snackMessage->getMessage('0').$e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function insertProducts($generalParams, $currentOrder, $assembledProducts){
        try {
            $nrSeqProdCup = $this->util->newCode('ITCOMANDAVEN'.$generalParams['CDFILIAL'], 6);
            foreach($assembledProducts as $product){
                $nrProdComVen = $this->util->newCode('ITCOMANDAVEN'.$generalParams['CDFILIAL'].$currentOrder['NRCOMANDA'], 6);
                $idStPrComVen = $this->getTheThing($generalParams['CDFILIAL'], $generalParams['CDLOJA'], $product['CDPRODUTO'], $generalParams['IDCONTROPROD']);

                $insertParams = array(
                    'CDFILIAL' => $generalParams['CDFILIAL'],
                    'NRVENDAREST' => $currentOrder['NRVENDAREST'],
                    'NRCOMANDA' => $currentOrder['NRCOMANDA'],
                    'NRPRODCOMVEN' => $nrProdComVen,
                    'CDPRODUTO' => $product['CDPRODUTO'],
                    'QTPRODCOMVEN' => $product['QTDPRODUTO'],
                    'VRPRECCOMVEN' => $product['PRECO'],
                    'TXPRODCOMVEN' => $product['TXPRODCOMVEN'],
                    'IDSTPRCOMVEN' => $idStPrComVen,
                    'VRDESCCOMVEN' => $product['DESC'],
                    'NRLUGARMESA' => '1',
                    'IDPRODIMPFIS' => 'N',
                    'CDLOJA' => $generalParams['CDLOJA'],
                    'VRACRCOMVEN' => $product['ACRE'],
                    'NRSEQPRODCOM' => null,
                    'NRSEQPRODCUP' => $nrSeqProdCup,
                    'VRPRECCLCOMVEN' => 0,
                    'CDCAIXACOLETOR' => $generalParams['CDCAIXA'],
                    'CDPRODPROMOCAO' => null,
                    'CDVENDEDOR' => null
                );
                $this->productService->insertProduct($insertParams);
            }
        } catch (\Exception $e) {
            throw new Exception\SqlException($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

    private function processObservations($product){
        if (isset($product['detailPages'][0]['selectItems']) || array_key_exists('selectItems', $product['detailPages'][0])){
            $TXPRODCOMVEN = '';
            foreach ($product['detailPages'][0]['selectItems'] as $observation){
                $TXPRODCOMVEN .= $observation['desc'] . '; ';
            }
        }
        else {
            return null;
        }
    }

    public function getTheThing($CDFILIAL, $CDLOJA, $CDPRODUTO, $IDCONTROPROD){
        try {
            $theThing = $this->productService->getSectorCode($CDFILIAL, $CDLOJA, $CDPRODUTO);
            if ($IDCONTROPROD === 'S' && $theThing !== null){
                $finalThing = '1';
            }
            else if ($IDCONTROPROD === 'N'){
                $finalThing = '4';
            }
            else {
                $finalThing = $IDCONTROPROD;
            }
            return $finalThing;
        } catch (\Exception $e) {
            throw new Exception\SqlException($this->snackMessage->getMessage('0').$e->getMessage());
        }
    }

}