<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\CreditCard\CreditCardTransaction;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Phone;
use Zeedhi\PaymentPlatform\Item;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyPlatform;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;

class Recharge extends \Zeedhi\Framework\Controller\Simple {

    protected $instanceManager;
    protected $rechargeService;
    protected $snackMessage;
    protected $util;
    protected $bancoService;
    protected $vendaOnlineservice;
    protected $smtpService;
    protected $balanceService;
    protected $profileService;
    protected $entityManager;
    protected $evoluiSaldoService;

    public function __construct(\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Service\Recharge $rechargeService, \Util\SnackMessage $snackMessage, \Util\Util $util,\Service\Banco $bancoService, \Service\ParaVendaOnline $vendaOnlineservice, \Service\SMTP $smtpService, \Service\Balance $balanceService, \Service\Profile $profileService, \Doctrine\ORM\EntityManager $entityManager, \Service\EvoluiSaldo $evoluiSaldoService){
        $this->instanceManager = $instanceManager;
        $this->rechargeService = $rechargeService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
        $this->bancoService = $bancoService;
        $this->vendaOnlineservice = $vendaOnlineservice;
        $this->smtpService = $smtpService;
        $this->balanceService = $balanceService;
        $this->profileService = $profileService;
        $this->entityManager = $entityManager;
        $this->evoluiSaldoService = $evoluiSaldoService;
    }

    public function decodeFlag($flag) {
        switch($flag){
            case 1:
                return Creditcard::FLAG_VISA;
            case 2:
                return Creditcard::FLAG_MASTERCARD;
            case 3:
                return Creditcard::FLAG_AMEX;
            case 5:
                return Creditcard::FLAG_HIPERCARD;
            case 6:
                return Creditcard::FLAG_AURA;
            case 33:
                return Creditcard::FLAG_DINERS;
            case 41:
                return Creditcard::FLAG_ELO;
            case 224:
                return Creditcard::FLAG_ALELO;
            case 281:
                return Creditcard::FLAG_SODEXO;
        }
    }

    public function getRechargeGrid(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL = $params[0]['value'];
            $NRCPFRESPCON = $params[1]['value'];
            $CDCLIENTE = $params[2]['value'];

            // Checks if consumer details are complete.
            $consumerValidation = $this->rechargeService->checkConsumerDetails($CDCLIENTE, $NRCPFRESPCON);

            if ($consumerValidation){
                $grid = array("V" => true);
            }
            else {
                $grid = $this->rechargeService->getRechargeGrid($CDFILIAL, $NRCPFRESPCON, $CDCLIENTE);
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeGridRepository', $grid));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function paymentServiceExists(){
        if (class_exists("\Payment\PaymentService")){
            return true;
        } else {
            throw new \Exception($this->snackMessage->getMessage('9'));
        }
    }

    private function verifyConsumerData($consumerData){

        if (Empty($consumerData['NMRESPCONS'])){
            throw new \Exception($this->snackMessage->getMessage('10'));
        }

        if (Empty($consumerData['NRCPFRESPCON'])){
            throw new \Exception($this->snackMessage->getMessage('11'));
        }

        if (Empty($consumerData['DSEMAILCONS'])){
            throw new \Exception($this->snackMessage->getMessage('29'));
        }

        if (Empty($consumerData['NRTELECONS'])){
            throw new \Exception($this->snackMessage->getMessage('30'));
        }

        $enderecoLabels = array(
            'DSENDECONS',
            'NMPAIS',
            'SGESTADO',
            'NMMUNICIPIO',
            'NRCEPCONS',
            'NMBAIRRO',
            'NRENDECONS'
        );

        foreach ($enderecoLabels as $label){
            if (Empty($consumerData[$label])){
                //Endereço incompleto
                throw new \Exception($this->snackMessage->getMessage('12'));
            }
        }
    }

    public function doBilletRecharge(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL       = $params[0]['value'];
            $CDCLIENTE      = $params[1]['value'];
            $consumidores   = $params[2]['value'];
            $banco          = $params[3]['value'];
            $valores        = $params[4]['value'];
            $bank           = json_decode($params[5]['value'], true);
            $valorTotal     = $params[6]['value'];
            $logoEmailInfo  = array(
                'appUrl'    => $params[7]['value'],
                'logoEmail' => $params[8]['value']
            );

            array_pop($valores); // Pulling out valorTotal.

            $filialData      = $this->rechargeService->getFilialData($CDFILIAL);
            $filialEnde      = $this->rechargeService->getFilialEndereco($CDFILIAL);
            $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $consumidores[0]);
            $NRBOLETO        = $this->setBilletNumber($bank);

            $this->verifyConsumerData($consumerDetails);

            $dadosBoleto = array(
                'prazoPagamento'  => $bank['QTDIASVENVND'],
                'valor'           => $valorTotal,
                'nossoNumero'     => $NRBOLETO,
                'numeroDocumento' => $NRBOLETO,
                'codigoCedente'   => $bank['DSINSCEMPVND']
            );

            $dadosCliente = array(
                'nomeCliente'  => $consumerDetails['NMRESPCONS'],
                'endereco1'    => $consumerDetails['DSENDECONS'] . ' ' . $consumerDetails['NMBAIRCONS'] . '' . $consumerDetails['NRCEPCONS'],
                'endereco2'    => $consumerDetails['NMMUNICIPIO'] . '/' . $consumerDetails['SGESTADO']
            );

            $dadosLoja = array(
                'nomeFantasia' => $filialData['NMFILIAL'],
                'siteCliente'  => $filialEnde['DSEMAILFILI'], //PARAM GERAL DSEMAISACVND
                'cpfCnpj'      => $filialData['NRINSJURFILI'],
                'endereco'     => $filialEnde['DSENDEFILI'] . ' ' . $filialEnde['NMBAIRFILI'],
                'cidadeEstado' => $filialEnde['NMMUNICIPIO'] . '/' . $filialEnde['SGESTADO'],
                'razaoSocial'  => $filialData['NMRAZSOCFILI'],
                'convenio'     => $bank['CDBANCOCONVVND'],
                'contrato'     => $bank['CDCONTRATOVND'],
                'conta' => array(
                    'agencia'       => $bank['CDAGENCIAVND'],
                    'digitoAgencia' => $bank['DIGCDAGENCIAVND'],
                    'conta'         => $bank['CDCONTCORR'],
                    'digitoConta'   => $bank['DIGCDCONTCORR']
                )
            );

            $alunos = '';
            foreach($consumidores as $consumidor){
                $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $consumidor);
                if($alunos == ''){
                    $alunos = $consumerDetails['NMCONSUMIDOR'];
                } else {
                    $alunos = $alunos . ', '. $consumerDetails['NMCONSUMIDOR'];
                }
            }

            $obs = "Credito no cartao do(s) aluno(s): " . $alunos;


            if ($this->paymentServiceExists()){
                $paymentService = new \Payment\PaymentService();
                $paymentResponse = $paymentService->generateBillet($bank['CDBANCCNAB'], $dadosBoleto, $dadosCliente, $dadosLoja, $obs, $NRBOLETO);

                if ($paymentResponse['error'] == null){

                    // Envia boleto por email.
                    try {
                        $this->billetEmail($consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $NRBOLETO, $logoEmailInfo);
                    } catch (\Exception $e){
                        $response->addMessage(new Message($this->snackMessage->getMessage('26'))); // Não foi possível enviar o boleto por email.
                    }

                    for ($i = 0; $i < sizeof($consumidores); $i++){
                        $NRPEDCRECONS = $this->util->newCode('PEDICREDCONS', 20);
                        $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $consumidores[$i]);
                        $this->rechargeService->insertPendentCredit(
                            $NRPEDCRECONS,
                            floatval(str_replace(',', '.', $valores[$i]['VRSALDCONFAM'])),
                            $consumerDetails['CDCLIENTE'],
                            $consumerDetails['CDCONSUMIDOR'],
                            $NRBOLETO
                        );

                        $valueToRecharge = floatval(str_replace(',', '.', $valores[$i]['VRSALDCONFAM']));
                        $this->rechargeService->insertPendentCreditFamily(
                            $NRPEDCRECONS,
                            $valores[$i]['CDFAMILISALD'],
                            $valueToRecharge
                        );
                    }

                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeBilletRepository', array('NRPEDCRECONS' => $NRBOLETO)));
                } else {
                    $response->addMessage(new Message($this->snackMessage->getMessage('26') . $paymentResponse['error']));
                }
            } else {
                $response->addMessage(new Message($this->snackMessage->getMessage('9')));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function setBilletNumber($bank){
        if ($bank['CDBANCCNAB'] === '399'){                               // HSBC
            $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 13); // HSBC
        }
        else if ($bank['CDBANCCNAB'] === '033') {                         // Santander
            $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 12); // Santander
        }
        else if ($bank['CDBANCCNAB'] === '237') {                         // Bradesco
            $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 11); // Bradesco
        }
        else if ($bank['CDBANCCNAB'] === '341') {                         // Itaú
            $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 8);  // Itaú
        }
        else if ($bank['CDBANCCNAB'] === '001') {                         // Banco do Brasil
            if (strlen($bank['CDBANCOCONVVND']) == 8){
                $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 9);
            }
            else if (strlen($bank['CDBANCOCONVVND']) == 7){
                $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 10);
            }
            else if (strlen($bank['CDBANCOCONVVND']) == 6){
                $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 5);
            }
        }
        else {
            $NRBOLETO = $this->util->newCode('PEDICREDCONSNRBOLETO', 13); // Default
        }

        return $NRBOLETO;
    }

    private function billetEmail($CDCLIENTE, $CDCONSUMIDOR, $NRPEDCRECONS, $logoEmailInfo){
        $jsonFile = file_get_contents(realpath(__DIR__ . "/../../../mobile/config.json"));
        $jsonFile = json_decode($jsonFile, true);

        // PDF conversion comented for now
        //$htmlBillet = realpath(__DIR__ . '/../../../' . $jsonFile['caminhoBoletos'] . '/' . $NRPEDCRECONS . '.html');
        //$pdfBillet = realpath(__DIR__ . '/../../../' . $jsonFile['caminhoBoletos'] . '/');
        //Generating pdf in phantomjs
        //shell_exec('phantomjs ' . __DIR__ . '/../../../mobile/js/controllers/billetEmailScript.js ' . $htmlBillet . ' ' . $pdfBillet . '/' . $NRPEDCRECONS . '.pdf');
        //$pdfBillet = realpath(__DIR__ . '/../../../' . $jsonFile['caminhoBoletos'] . '/' . $NRPEDCRECONS . '.pdf');

        $consumerDetails = $this->rechargeService->getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR);

        $billetUrl = $jsonFile['diretorioBoletos'] . $NRPEDCRECONS . '.html';
        $emailDestinatario = $consumerDetails['DSEMAILCONS'];

        $subject  = $this->snackMessage->getMessage('27');
        $sender = $this->snackMessage->getMessage('14');
        $content = '<center>'.$this->snackMessage->getMessage('23').'<br>'.$billetUrl.'</center>';

        try {
            $this->smtpService->sendEmail($consumerDetails['NMRESPCONS'], $emailDestinatario, $sender, $subject, $content, $logoEmailInfo);
        } catch (\Exception $e) {
            throw new \Exception($this->snackMessage->getMessage('26'));
        }
    }

    public function getBankData(Request\Filter $request, Response $response){
        try {

            $bancos = $this->bancoService->buscaBancos();
            $esitefDetails = $this->rechargeService->getEsitefDetails();
            $bandESiTef = $this->rechargeService->buscaBandeirasEsiTef();

            if (empty($bancos) && empty($esitefDetails)){
                $response->addMessage(new Message($this->snackMessage->getMessage('17')));
            } else {
                foreach ($bancos as &$banco){
                    $banco = $this->mascara($banco, $bancos[0]['CDPICTAGEN'], 'CDAGENCIAVND');
                    $banco = $this->mascara($banco, $bancos[0]['CDPICTCONT'], 'CDCONTCORR');
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('EsitefRepository', $esitefDetails));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BankDataRepository', $bancos));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaBandeirasesitefRepo', $bandESiTef));

        } catch (\Exception $e){
             $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function mascara($array, $mascara, $campo){
        $ndigitos = strpos($mascara, '-');

        if (($ndigitos) == false){
            $contaAtual = $array[$campo];
            $array['DIG'.$campo] = "";
        }
        else {
            $contaAtual = $array[$campo];
            $array[$campo] = substr($contaAtual, 0, $ndigitos);
            $array['DIG'.$campo]= substr($contaAtual, ($ndigitos));
        }
        return $array;
    }

    public function loadBandeirasEsiTef(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDFILIAL = $params[0]['value'];

            $bandESiTef = $this->rechargeService->loadBandeirasPorFilialEsiTef($CDFILIAL);
            if (empty($bandESiTef)) {
                $bandESiTef = $this->rechargeService->buscaBandeirasEsiTef();

                // This function is run during the login, even when the user isn't going to use credit card payment.
                // SO WHY IS IT SHOWING AN ERROR IF THE CLIENT HAS NO E-SITEF CODE?!
                // if (empty($bandESiTef)) {
                //     $response->addMessage(new Message($this->snackMessage->getMessage('17')));
                // }
            }
            //$bandESiTefPorFilial = $this->rechargeService->loadBandeirasPorFilialEsiTef($CDFILIAL);
            $bandESiTefFinal = [];
            $bandESiTefAux = [];
            foreach ($bandESiTef as $bandeira){
                $bandESiTefAux['NMBANDEIRA'] = $bandeira['NMBANDEIRA'];
                $bandESiTefAux['IDESITEF'] = [
                    'CDBANDEIRA' => $bandeira['IDESITEF'],
                    'CDTIPORECE' => $bandeira['CDTIPORECE']
                ];
                array_push($bandESiTefFinal, $bandESiTefAux);
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaBandeirasesitefRepo', $bandESiTefFinal));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    // Old payment
    /*public function doCardRecharge(Request\Filter $request, Response $response){
        try {

            $params = $request->getFilterCriteria()->getConditions();

            $valorPedido                 = $params[0]['value'];
            $bandeira                    = $params[1]['value'];
            $validadeCartao              = $params[2]['value'];
            $numeroCartao                = $params[3]['value'];
            $codigoSegurancaCartao       = $params[4]['value'];
            $consumer['CDCLIENTE']       = $params[5]['value'];
            $consumer['CDCONSUMIDOR']    = $params[6]['value'];
            $rechargeValues              = $params[7]['value'];
            $CDFILIAL                    = $params[8]['value'];
            $appUrl                      = $params[9]['value'];
            $logoEmail                   = $params[10]['value'];

            $consumerDetails = $this->profileService->getConsumerDetails($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);

            // if the consumer doesn't have any address, we get the country of the store,
            // to get the states for the address update
            if (empty($consumerDetails[0]['CDPAIS'])) {
            	$consumerDetails[0]['CDPAIS'] = $this->profileService->getFilialPais($CDFILIAL);
            }

            $consumerDetails[0]['DTNASCCONS'] = substr($consumerDetails[0]['DTNASCCONS'], 0, 10);

            $cpf = $consumerDetails[0]['NRCPFRESPCON'];
            if (!empty($cpf)){
                $cpf = substr($cpf, 0, 3).'.'.substr($cpf, 3, 3).'.'.substr($cpf, 6, 3).'-'.substr($cpf, 9, 2);
                $consumerDetails[0]['NRCPFRESPCON'] = $cpf;
            }

            $consumer['NMCONSUMIDOR']    = $consumerDetails[0]['NMCONSUMIDOR'];
            $consumer['NRCPFRESPCON']    = $consumerDetails[0]['NRCPFRESPCON'];
            $consumer['NMPAIS']          = $consumerDetails[0]['NMPAIS'];
            $consumer['SGESTADO']        = $consumerDetails[0]['SGESTADO'];
            $consumer['NMMUNICIPIO']     = $consumerDetails[0]['NMMUNICIPIO'];
            $consumer['DSENDECONS']      = $consumerDetails[0]['DSENDECONS'];
            $consumer['NMBAIRCONS']      = $consumerDetails[0]['NMBAIRCONS'];
            $consumer['NRCEPCONS']       = $consumerDetails[0]['NRCEPCONS'];
            $consumer['NRTELECONS']      = $consumerDetails[0]['NRTELECONS'];
            $consumer['NRENDECONS']      = $consumerDetails[0]['NRENDECONS'];
            $consumer['DSCOMPLENDECONS'] = $consumerDetails[0]['DSCOMPLENDECONS'];
            $consumer['DSEMAILCONS']     = $consumerDetails[0]['DSEMAILCONS'];

            if ($bandeira){
                $rechargeValues = json_decode($rechargeValues, true);

                $logoEmpresaInfo = array(
                    'appUrl'      => $appUrl,
                    'logoEmail' => $logoEmail
                );

                $NRTRANSACAO = $this->util->newCode('TRANSACAO', 12);

                // Call method to insert into table TRANSACAO
                $transacaoParams = array(
                    'NRTRANSACAO'  => $NRTRANSACAO,
                    'DSTRANSACAO'  => "Transacao iniciada.",
                    'CDCLIENTE'    => $consumer['CDCLIENTE'],
                    'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
                    'VRMOVEXTCONS' => $valorPedido
                );
                $this->rechargeService->insertTransacao($transacaoParams);

                $controlArray = array();
                foreach ($rechargeValues as $recharge){
                    if (!($this->consumerAlreadyRegistered($consumer['CDCONSUMIDOR'], $controlArray))){
                        $transactionConsParams = array (
                            'NRTRANSACAO' => $NRTRANSACAO,
                            'CDCLIENTE' => $consumer['CDCLIENTE'],
                            'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR']
                        );
                        $this->rechargeService->insertTransacaoCons($transactionConsParams);
                    }
                    array_push($controlArray, $consumer['CDCONSUMIDOR']);
                }

                $expMonth   = substr($validadeCartao, 0 , 2);
                $extYear    = substr($validadeCartao, 2);

                $onlineParams = $this->rechargeService->getOnlinePaymentInfo($CDFILIAL);

                $paymentPlatform = $this->rechargeService->getPaymentPlatform($CDFILIAL);
                $paymentPlatform = $paymentPlatform['IDPLATAFORMAPAG'];

                if($paymentPlatform === "1") {  
                    $client = \Zeedhi\PaymentPlatform\Implementations\MundiPagg\PaymentImpl::factory($this->util->getXMLParameter('merchant_key'));
                }
                if($paymentPlatform === "2") {
                    $extYear = '20'.$extYear;

                    if($bandeira === 'Mastercard'){
                        $bandeira = 'Master';
                    }

                    $client = \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::factory(
                        $onlineParams['CDLOJAESITEF'],
                        $onlineParams['IDLOJAESITEF'],
                        \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::PRODUCTION
                    );
                }
                if($paymentPlatform === "3") {
                    $client = \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::factory(
                        $onlineParams['CDLOJAESITEF'],
                        $onlineParams['IDLOJAESITEF'],
                        \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::PRODUCTION
                    );
                }
                if($paymentPlatform === "4"){
                    $client = \Zeedhi\PaymentPlatform\Implementations\PagarMe\PaymentImpl::factory(
                        $onlineParams['CDLOJAESITEF']
                    );
                }

                $customer = new Customer();
                $customer->setName($consumer['NMCONSUMIDOR']);

                if($paymentPlatform === "4"){
                    $consumer['NRCEPCONS'] = str_replace('-', '', $consumer['NRCEPCONS']);
                    $address = new Address($consumer['DSENDECONS'],
                                           $consumer['NRENDECONS'],
                                           $consumer['DSCOMPLENDECONS'],
                                           $consumer['NMBAIRCONS'],
                                           $consumer['NMMUNICIPIO'],
                                           $consumer['SGESTADO'],
                                           $consumer['NRCEPCONS'],
                                           'br');

                    $customer->setId($consumer['CDCLIENTE'].$consumer['CDCONSUMIDOR']);
                    $customer->setEmail($consumer['DSEMAILCONS']);
                    $consumer['NRCPFRESPCON'] = str_replace('.', '', $consumer['NRCPFRESPCON']);
                    $consumer['NRCPFRESPCON'] = str_replace('-', '', $consumer['NRCPFRESPCON']);
                    $customer->setDocument($consumer['NRCPFRESPCON']);
                    $customer->setPhone('+55'.$consumer['NRTELECONS']);
                    $customer->setAddress($address);
                }

                $creditCard = new CreditCard(
                    $consumer['NMCONSUMIDOR'],
                    $numeroCartao,
                    $expMonth,
                    $extYear,
                    $codigoSegurancaCartao,
                    $this->decodeFlag($bandeira)
                );
                
                $items = array();

                $item1 = new Item('1', 'Recarga No Cash', ceil($valorPedido * 100), 1);
                array_push($items, $item1);
                
                $transaction = new NonInstantBuyTransaction(ceil($valorPedido * 100), $NRTRANSACAO, $creditCard, $customer);
                $transaction->setItems($items);
                $transactionKey = $client->nonInstantBuyTransaction($transaction);

                if($paymentPlatform === "2"){
                    if($transactionKey['status'] != "00" && $transactionKey['status'] != "000" && $transactionKey['status'] != "11" && $transactionKey['status'] != "4" && $transactionKey['status'] != "6"){
                        throw new \Exception('Não Autorizada');
                    }
                    if($transactionKey['status'] === "57"){
                        throw new \Exception('Cartão Expirado');
                    }
                    if($transactionKey['status'] === "78"){
                        throw new \Exception('Cartão Bloqueado');
                    }
                    if($transactionKey['status'] === "99"){
                        throw new \Exception('Time Out');
                    }
                    if($transactionKey['status'] === "77"){
                        throw new \Exception('Cartão Cancelado');
                    }
                    if($transactionKey['status'] === "70"){
                        throw new \Exception('Problemas com o Cartão de Crédito');
                    }
                }

                if($paymentPlatform === "4"){
                    if($transactionKey['status'] !== 'authorized'){
                        if($transactionKey['message'] === 'acquirer'){
                            throw new \Exception('Transação não autorizada: dados do cartão inválidos.');
                        } else if($transactionKey['message'] === 'acquirer_timeout'){
                            throw new \Exception('Transação não autorizada: sem resposta da adquirente.');
                        } else{
                            throw new \Exception('Transação não autorizada: '.$transactionKey['message']);
                        }
                    }
                }
                
                $rechargeConsumersArray = array();

                foreach ($rechargeValues as $family) {
                    $isSetConsumer = false;
                    
                    forEach($rechargeConsumersArray as $rechargeConsumer){
                        if($rechargeConsumer === $family['NMCONSUMIDOR']){
                            $isSetConsumer = true;
                        }
                    }
                    
                    if(!$isSetConsumer){
                        array_push($rechargeConsumersArray, $family['NMCONSUMIDOR']);
                    }
                    
                    $isFirstInsertion = $this->rechargeService->isFirstInsertion($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $currentBalance = $this->rechargeService->getCurrentBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                    $valueToRecharge = floatval(str_replace(',', '.', $family['VRSALDCONFAM']));

                    // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                    $balance = $this->balanceService->checkBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                    $VRSALDCONEXT = $valueToRecharge;
                    //If it's not first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
                    if(!empty($isFirstInsertion)){
                        //Add VRSALDCONEXT to previous balance
                        if(!empty($currentBalance)){
                            $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                        }
                    }
                    $this->rechargeService->insertExtratoCons($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito No Cash (Cartao)', $valueToRecharge, 'C', $VRSALDCONEXT);

                    //If there is no record in SALDOCONS, we have to create one with current valueToRecharge
                    if (empty($balance)){
                        // Without any balance, the balance can't be digivolved, so we must INSERT ADDITIONAL BALANCES!
                        $this->balanceService->createBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $valueToRecharge);
                    } else {
                        //If there is balance already, we have to update it with the new value
                        $this->balanceService->updateBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                    }
                }
                $result = true;
                
                $rechargeConsumers = '';
            
                forEach($rechargeConsumersArray as $rechargeConsumer){
                    $rechargeConsumers = $rechargeConsumers.$rechargeConsumer.', ';
                }
                
                $rechargeConsumers = substr($rechargeConsumers, 0, -2);

                // Update TRANSACAO case success.
                if($paymentPlatform === "2" || $paymentPlatform === "4"){
                    $nsu = $transactionKey['nsu'];
                } else{
                    $nsu = $transactionKey;
                }

                $transacaoParams = array(
                    'NRTRANSACAO' => $NRTRANSACAO,
                    'DSTRANSACAO' => 'Transação feita com sucesso.',
                    'NRNIT'       => $nsu
                );
                $this->rechargeService->updateTransacao($transacaoParams);

                if($paymentPlatform === '2' || $paymentPlatform === '4'){
                    $transactionKey = $client->captureTransaction($transactionKey['captureUrl'], $transactionKey['paymentId']);
                }

                // E-mail - The consumerDetails is the same for all the consumers with the same CDCLIENTE, so we don't need to build a foreach to each one.
                $codigoCartao = 'XXXX XXXX XXXX '.substr($numeroCartao, 12);

                // Formating values to send e-mail
                $valorPedido = str_replace(".", "", $valorPedido);
                $valorPedido = substr($valorPedido, 0, (strlen($valorPedido)-2)).','.substr($valorPedido, -2, strlen($valorPedido));
                $temp        = $valorPedido.substr($valorPedido, 2);

                $nomeDestinatario = $consumer['NMCONSUMIDOR'];
                $emailDestinatario = $consumer['DSEMAILCONS'];

                // Creating beautiful e-mail template.
                $subject = 'Comprovante de recarga do cartão';
                $sender  = 'No Cash';
                $content =
                    '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
                    '<b>Nome do(s) Aluno(s): </b>'.$rechargeConsumers.'<br>'.
                    '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
                    '<p><b>Valor Total da Recarga: </b>R$ '.$valorPedido.'</p>'.
                    '<p><b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y") .'</p>'
                ;

                try {
                    // Sends the e-mail!
                    $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
                    $mailSent = true;
                } catch (\Exception $e){
                    $mailSent = false;
                }

                /* *** E-MAIL SENT *** 
                $result = true; // Indicates everything was done successfully.
                if ($mailSent) $response->addMessage(new Message('Recarga concluída com sucesso!'));
                else $response->addMessage(new Message('Recarga concluída com sucesso! Não foi possível enviar o e-mail de confirmação.'));

                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
            }
        } catch (\Exception $e){
            // Something went wrong with the payment.
            $message = substr($e->getMessage(),0,255);
            // Update TRANSACAO case failure.
            $transacaoParams = array(
                'NRTRANSACAO' => $NRTRANSACAO,
                'DSTRANSACAO' => $message,
                'NRNIT' => null
            );
            $this->rechargeService->updateTransacao($transacaoParams);

            foreach ($rechargeValues as $family){
                $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                $CDCLIENTENOTIFAPP = $consumer['CDCLIENTE'];
                $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                $CDTIPONOTIFICAAPP = '003';

                $this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
            }

            $response->addMessage(new Message($message));
            $result = false; // Indicates something went wrong.
            $response->addMessage(new Message($e->getMessage()));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
        }
    }*/

    //New Payment (GRSA treatment)
    public function doCardRecharge(Request\Filter $request, Response $response){
        try {

            $params = $request->getFilterCriteria()->getConditions();

            $valorPedido                 = $params[0]['value'];
            $bandeira                    = $params[1]['value'];
            $tipoRece                    = $params[2]['value'];
            $validadeCartao              = $params[3]['value'];
            $numeroCartao                = $params[4]['value'];
            $codigoSegurancaCartao       = $params[5]['value'];
            $consumer['CDCLIENTE']       = $params[6]['value'];
            $consumer['CDCONSUMIDOR']    = $params[7]['value'];
            $rechargeValues              = $params[8]['value'];
            $CDFILIAL                    = $params[9]['value'];
            $appUrl                      = $params[10]['value'];
            $logoEmail                   = $params[11]['value'];
            $nomeCartao                  = $params[12]['value'];

            $consumerDetails = $this->profileService->getConsumerDetails($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);

            // if the consumer doesn't have any address, we get the country of the store,
            // to get the states for the address update
            if (empty($consumerDetails[0]['CDPAIS'])) {
            	$consumerDetails[0]['CDPAIS'] = $this->profileService->getFilialPais($CDFILIAL);
            }

            $consumerDetails[0]['DTNASCCONS'] = substr($consumerDetails[0]['DTNASCCONS'], 0, 10);

            $cpf = $consumerDetails[0]['NRCPFRESPCON'];
            if (!empty($cpf)){
                $cpf = substr($cpf, 0, 3).'.'.substr($cpf, 3, 3).'.'.substr($cpf, 6, 3).'-'.substr($cpf, 9, 2);
                $consumerDetails[0]['NRCPFRESPCON'] = $cpf;
            }

            $consumer['NMCONSUMIDOR']    = $consumerDetails[0]['NMCONSUMIDOR'];
            $consumer['NRCPFRESPCON']    = $consumerDetails[0]['NRCPFRESPCON'];
            $consumer['NMPAIS']          = $consumerDetails[0]['NMPAIS'];
            $consumer['SGESTADO']        = $consumerDetails[0]['SGESTADO'];
            $consumer['NMMUNICIPIO']     = $consumerDetails[0]['NMMUNICIPIO'];
            $consumer['DSENDECONS']      = $consumerDetails[0]['DSENDECONS'];
            $consumer['NMBAIRCONS']      = $consumerDetails[0]['NMBAIRCONS'];
            $consumer['NRCEPCONS']       = $consumerDetails[0]['NRCEPCONS'];
            $consumer['NRTELECONS']      = $consumerDetails[0]['NRTELECONS'];
            $consumer['NRCELULARCONS']   = $consumerDetails[0]['NRCELULARCONS'];
            $consumer['NRENDECONS']      = $consumerDetails[0]['NRENDECONS'];
            $consumer['DSCOMPLENDECONS'] = $consumerDetails[0]['DSCOMPLENDECONS'];
            $consumer['DSEMAILCONS']     = $consumerDetails[0]['DSEMAILCONS'];
            $consumer['CDDDDMUNICIP']    = $consumerDetails[0]['CDDDDMUNICIP'];
            $consumer['HABENVEMAIL']     = $consumerDetails[0]['HABENVEMAIL'];
            $consumer['CDTIPOCONS']      = $consumerDetails[0]['CDTIPOCONS'];

            if ($bandeira){
                $rechargeValues = json_decode($rechargeValues, true);

                $logoEmpresaInfo = array(
                    'appUrl'      => $appUrl,
                    'logoEmail' => $logoEmail
                );

                $NRTRANSACAO = $this->util->newCode('TRANSACAO', 12);

                // Call method to insert into table TRANSACAO
                $transacaoParams = array(
                    'NRTRANSACAO'  => $NRTRANSACAO,
                    'DSTRANSACAO'  => "Transacao digitada iniciada.",
                    'CDCLIENTE'    => $consumer['CDCLIENTE'],
                    'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
                    'VRMOVEXTCONS' => $valorPedido
                );
                $this->rechargeService->insertTransacao($transacaoParams);

                $transactionConsParams = array (
                    'NRTRANSACAO' => $NRTRANSACAO,
                    'CDCLIENTE' => $consumer['CDCLIENTE'],
                    'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR']
                );
                $this->rechargeService->insertTransacaoCons($transactionConsParams);

                try{
                    $expMonth   = substr($validadeCartao, 0 , 2);
                    $extYear    = substr($validadeCartao, 2);

                    $onlineParams = $this->rechargeService->getOnlinePaymentInfo($CDFILIAL);

                    $paymentPlatform = $this->rechargeService->getPaymentPlatform($CDFILIAL);
                    $paymentPlatform = $paymentPlatform['IDPLATAFORMAPAG'];

                    if($paymentPlatform === "1") {  
                        $client = \Zeedhi\PaymentPlatform\Implementations\MundiPaggPHP\PaymentImpl::factory($onlineParams['CDLOJAESITEF'], $onlineParams['IDLOJAESITEF']);
                    }
                    if($paymentPlatform === "2") {
                        $extYear = '20'.$extYear;

                        if($bandeira === 'Mastercard'){
                            $bandeira = 'Master';
                        }

                        $client = \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::factory(
                            $onlineParams['CDLOJAESITEF'],
                            $onlineParams['IDLOJAESITEF'],
                            \Zeedhi\PaymentPlatform\Implementations\Cielo\PaymentImpl::PRODUCTION
                        );
                    }
                    if($paymentPlatform === "3") {
                        $client = \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::factory(
                            $onlineParams['CDLOJAESITEF'],
                            $onlineParams['IDLOJAESITEF'],
                            \Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl::PRODUCTION
                        );
                    }
                    if($paymentPlatform === "4"){
                        $client = \Zeedhi\PaymentPlatform\Implementations\PagarMe\PaymentImpl::factory(
                            $onlineParams['CDLOJAESITEF']
                        );
                    }

                    $customer = new Customer();
                    $creditCard = new CreditCard();
                    $customer->setName($consumer['NMCONSUMIDOR']);

                    if($paymentPlatform === "4" || $paymentPlatform === "1"){
                        $consumer['NRCEPCONS'] = str_replace('-', '', $consumer['NRCEPCONS']);
                        $address = new Address($consumer['DSENDECONS'],
                                            $consumer['NRENDECONS'],
                                            $consumer['DSCOMPLENDECONS'],
                                            null,
                                            $consumer['NMMUNICIPIO'],
                                            $consumer['SGESTADO'],
                                            $consumer['NRCEPCONS'],
                                            'br',
                                            "");

                        $customer->setId($consumer['CDCLIENTE'].$consumer['CDCONSUMIDOR']);
                        $customer->setEmail($consumer['DSEMAILCONS']);
                        $consumer['NRCPFRESPCON'] = str_replace('.', '', $consumer['NRCPFRESPCON']);
                        $consumer['NRCPFRESPCON'] = str_replace('-', '', $consumer['NRCPFRESPCON']);
                        $customer->setDocument($consumer['NRCPFRESPCON']);
                        $customer->setAddress($address);
                    }

                    if($paymentPlatform === "4"){
                        $customer->setPhone('+55'.$consumer['NRCELULARCONS']);
                    }

                    if($paymentPlatform === "1"){
                        $option = new \MundiAPILib\Models\CreateCardOptionsRequest();
                        $option->verifyCard = true;
                        
                        $phoneNumber = str_replace('(', '', $consumer['NRCELULARCONS']);
                        $phoneNumber = str_replace(')', '', $phoneNumber);
                        $phoneNumber = str_replace('-', '', $phoneNumber);
                        $phoneNumber = str_replace(' ', '', $phoneNumber);

                        $phone = new Phone ("31",
                                            "55",
                                            $phoneNumber
                                            );

                        $customer->setPhone($phone);
                        $customer->setMetadata("");
                        $customer->setCode($consumer['CDCONSUMIDOR']);
                        $customer->setType("INDIVIDUAL");

                        $creditCard->setBillingAddress($address);
                        $creditCard->setBillingAddressId("");
                        $creditCard->setMetadata("");
                        $creditCard->setType("INDIVIDUAL");
                        $creditCard->setOptions($option);
                        $creditCard->setHolderDocument($consumer['NRCPFRESPCON']);
                        $creditCard->setPrivateLabel(false);
                    }

                    $creditCard->setHolderName($nomeCartao);
                    $creditCard->setNumbers($numeroCartao);
                    $creditCard->setExpDateMonth($expMonth);
                    $creditCard->setExpDateYear($extYear);
                    $creditCard->setCvv($codigoSegurancaCartao);
                    $creditCard->setFlag($this->decodeFlag($bandeira));
                
                    $items = array();

                    $item1 = new Item('1', 'Recarga No Cash', floor($valorPedido * 100), 1);
                    array_push($items, $item1);

                    $transacaoParams = array(
                        'NRTRANSACAO' => $NRTRANSACAO,
                        'DSTRANSACAO' => "Processando pagamento de recarga digitada.",
                        'NRNIT' => null
                    );
                    $this->rechargeService->updateTransacao($transacaoParams);
                
                    $transaction = new NonInstantBuyTransaction(floor($valorPedido * 100), $NRTRANSACAO, $creditCard, $customer);
                    $transaction->setItems($items);
                    $transactionKey = $client->nonInstantBuyTransaction($transaction);
                    
                    if($paymentPlatform === "1" || $paymentPlatform === "2" || $paymentPlatform === "3" || $paymentPlatform === "4"){
                        $nsu = $transactionKey['nsu'];
                    } else{
                        $nsu = $transactionKey;
                    }

                    if($paymentPlatform === "2"){
                        if($transactionKey['status'] != "00" && $transactionKey['status'] != "000" && $transactionKey['status'] != "11" && $transactionKey['status'] != "4" && $transactionKey['status'] != "6"){
                            throw new \Exception('Não Autorizada');
                        }
                        if($transactionKey['status'] === "57"){
                            throw new \Exception('Cartão Expirado');
                        }
                        if($transactionKey['status'] === "78"){
                            throw new \Exception('Cartão Bloqueado');
                        }
                        if($transactionKey['status'] === "99"){
                            throw new \Exception('Time Out');
                        }
                        if($transactionKey['status'] === "77"){
                            throw new \Exception('Cartão Cancelado');
                        }
                        if($transactionKey['status'] === "70"){
                            throw new \Exception('Problemas com o Cartão de Crédito');
                        }
                    }

                    if($paymentPlatform === "4"){
                        if($transactionKey['status'] !== 'authorized'){
                            if($transactionKey['message'] === 'acquirer'){
                                throw new \Exception('Transação não autorizada: dados do cartão inválidos.');
                            } else if($transactionKey['message'] === 'acquirer_timeout'){
                                throw new \Exception('Transação não autorizada: sem resposta da adquirente.');
                            } else{
                                throw new \Exception('Transação não autorizada: '.$transactionKey['message']);
                            }
                        }
                    }

                    $transacaoParams = array(
                        'NRTRANSACAO' => $NRTRANSACAO,
                        'DSTRANSACAO' => "Transação digitada aprovada. Inserindo créditos.",
                        'NRNIT' => $nsu
                    );
                    $this->rechargeService->updateTransacao($transacaoParams);

                    $recalcParam = $this->instanceManager->getParameter("recalculateMethod");

                    try{
                        if($paymentPlatform === '1' || $paymentPlatform === '3'){
                            $this->entityManager->getConnection()->beginTransaction();
                        }

                        $rechargeConsumersArray = array();

                        foreach ($rechargeValues as $family) {
                            $isSetConsumer = false;
                            
                            forEach($rechargeConsumersArray as $rechargeConsumer){
                                if($rechargeConsumer === $family['NMCONSUMIDOR']){
                                    $isSetConsumer = true;
                                }
                            }
                            
                            if(!$isSetConsumer){
                                array_push($rechargeConsumersArray, $family['NMCONSUMIDOR']);
                            }
                            
                            $isFirstInsertion = $this->rechargeService->isFirstInsertion($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                            $currentBalance = $this->rechargeService->getCurrentBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);
                            $valueToRecharge = floatval(str_replace(',', '.', $family['VRSALDCONFAM']));

                            // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                            $balance = $this->balanceService->checkBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                            $VRSALDCONEXT = $valueToRecharge;
                            //If it's not first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
                            if(!empty($isFirstInsertion)){
                                //Add VRSALDCONEXT to previous balance
                                if(!empty($currentBalance)){
                                    $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                                }
                            }
                            $this->rechargeService->insertExtratoCons($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito NoCash (Cartao)', $valueToRecharge, 'C', $VRSALDCONEXT, $tipoRece, $CDFILIAL);

                            //If there is no record in SALDOCONS, we have to create one with current valueToRecharge
                            if (empty($balance)){
                                // Without any balance, the balance can't be digivolved, so we must INSERT ADDITIONAL BALANCES!
                                $this->balanceService->createBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $valueToRecharge);
                            } else {
                                //If there is balance already, we have to update it with the new value
                                $this->balanceService->updateBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                            }
                            
                            //Bonus recharge
                            $bonusRecharge = $this->balanceService->getBonusRecharge($CDFILIAL, $consumer['CDTIPOCONS'], $tipoRece);
                            
                            if(!empty($bonusRecharge)){
                                if(!empty($currentBalance)){
                                    $vrPrevConExt = $currentBalance[0]['VRSALDCONEXT'];
                                } else{
                                    $vrPrevConExt = 0;
                                }
                                if($valueToRecharge >= $bonusRecharge['VRCARGACREDITO'] && $vrPrevConExt <= $bonusRecharge['VRSALDCONSBONUS']){
                                    $VRSALDCONEXT += floatval($bonusRecharge['VRBONUSCREDITO']);
                                    $this->rechargeService->insertExtratoCons($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], 'Credito NoCash (Bonus)', $bonusRecharge['VRBONUSCREDITO'], 'C', $VRSALDCONEXT, $bonusRecharge['CDTIPORECE'], $CDFILIAL);
                                    $this->balanceService->updateBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
                                }   
                            }

                            //Racalculate balance process
                            if($recalcParam === 'F'){
                                $evoluiSaldoDate = $this->evoluiSaldoService->getTransactionDate($recalcParam, $consumer['CDCLIENTE'], $family['CDCONSUMIDOR']);
                                $NRORG = $this->evoluiSaldoService->getConsumerNrorg($consumer['CDCLIENTE'], $family['CDCONSUMIDOR']);
                                
                                $conditions = array(
                                    'CDCLIENTE' => $consumer['CDCLIENTE'],
                                    'CDCONSUMIDOR' => $family['CDCONSUMIDOR'],
                                    'DTMOVEXTCONS' => $evoluiSaldoDate['DTMOVEXTCONS'],
                                    'NRORG' => $NRORG['NRORG']
                                );

                                $updateSaldo = $this->evoluiSaldoService->EvoluiSaldoFunction($conditions);
                            } else if($recalcParam === 'P'){
                                $evoluiSaldoDate = $this->evoluiSaldoService->getTransactionDate($recalcParam, $consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD']);

                                $this->evoluiSaldoService->recalculateBalance($consumer['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $evoluiSaldoDate['DTMOVEXTCONS']);
                            }
                        }
                        $result = true;
                        
                        $rechargeConsumers = '';
                    
                        forEach($rechargeConsumersArray as $rechargeConsumer){
                            $rechargeConsumers = $rechargeConsumers.$rechargeConsumer.', ';
                        }
                        
                        $rechargeConsumers = substr($rechargeConsumers, 0, -2);

                        // Update TRANSACAO case success.

                        $transacaoParams = array(
                            'NRTRANSACAO' => $NRTRANSACAO,
                            'DSTRANSACAO' => 'Transação digitada finalizada com sucesso.',
                            'NRNIT'       => $nsu
                        );
                        $this->rechargeService->updateTransacao($transacaoParams);

                        if($paymentPlatform === "1" || $paymentPlatform === '3'){
                            $captureResponse = $client->captureTransaction($transactionKey['captureUrl'], $transactionKey['paymentId']);
                            $this->entityManager->getConnection()->commit();
                        } else if($paymentPlatform === '2' || $paymentPlatform === '4'){
                            $transactionKey = $client->captureTransaction($transactionKey['captureUrl'], $transactionKey['paymentId']);
                        }

                        // E-mail - The consumerDetails is the same for all the consumers with the same CDCLIENTE, so we don't need to build a foreach to each one.
                        $codigoCartao = 'XXXX XXXX XXXX '.substr($numeroCartao, 12);

                        // Formating values to send e-mail
                        $valorPedido = str_replace(".", "", $valorPedido);
                        $valorPedido = substr($valorPedido, 0, (strlen($valorPedido)-2)).','.substr($valorPedido, -2, strlen($valorPedido));
                        $temp        = $valorPedido.substr($valorPedido, 2);

                        $nomeDestinatario = $consumerDetails[0]['NMRESPCONS'];
                        $emailDestinatario = $consumer['DSEMAILCONS'];

                        // Creating beautiful e-mail template.
                        $subject = 'Comprovante de recarga do cartão';
                        $sender  = 'No Cash';
                        $content =
                            '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
                            '<b>Nome do(s) Aluno(s): </b>'.$rechargeConsumers.'<br>'.
                            '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
                            '<p><b>Valor Total da Recarga: </b>R$ '.$valorPedido.'</p>'.
                            '<p><b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y") .'</p>'
                        ;

                        try {
                            // Sends the e-mail!
                            if($consumer['HABENVEMAIL'] === 'S'){
                                $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content, $logoEmpresaInfo);
                            }
                            $mailSent = true;
                        } catch (\Exception $e){
                            $mailSent = false;
                        }

                        /* *** E-MAIL SENT *** */
                        $result = true; // Indicates everything was done successfully.
                        if ($mailSent) $response->addMessage(new Message('Recarga concluída com sucesso!'));
                        else $response->addMessage(new Message('Recarga concluída com sucesso! Não foi possível enviar o e-mail de confirmação.'));

                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
                    } catch(\Exception $e){
                        // Something went wrong with the credits update.
                        if($paymentPlatform === "1" || $paymentPlatform === '3'){
                            $this->entityManager->getConnection()->rollBack();
                        }
                        
                        $message = substr($e->getMessage(),0,255);
                        // Update TRANSACAO case failure.
                        $transacaoParams = array(
                            'NRTRANSACAO' => $NRTRANSACAO,
                            'DSTRANSACAO' => 'Erro na inserção de crédito da recarga digitada. Erro: ' . $message,
                            'NRNIT' => null
                        );
                        $this->rechargeService->updateTransacao($transacaoParams);

                        $rechargeValues = json_decode($params[8]['value'], true);

                        foreach ($rechargeValues as $family){
                            $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                            $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                            $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                            $CDCLIENTENOTIFAPP = $consumer['CDCLIENTE'];
                            $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                            $CDTIPONOTIFICAAPP = '003';

                            //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
                        }

                        $response->addMessage(new Message($message));
                        $result = false; // Indicates something went wrong.
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
                    }
                } catch(\Exception $e){
                    // Something went wrong with the payment.
                    $message = substr($e->getMessage(),0,255);
                    // Update TRANSACAO case failure.
                    $transacaoParams = array(
                        'NRTRANSACAO' => $NRTRANSACAO,
                        'DSTRANSACAO' => 'Erro no processamento do pagamento da recarga digitada. Erro: ' . $message,
                        'NRNIT' => null
                    );
                    $this->rechargeService->updateTransacao($transacaoParams);

                    $rechargeValues = json_decode($params[8]['value'], true);

                    foreach ($rechargeValues as $family){
                        $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                        $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                        $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                        $CDCLIENTENOTIFAPP = $consumer['CDCLIENTE'];
                        $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                        $CDTIPONOTIFICAAPP = '003';

                        //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
                    }

                    $response->addMessage(new Message($message));
                    $result = false; // Indicates something went wrong.
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
                }
            } else{
                throw new \Exception('Bandeira não reconhecida!');
            }
        } catch (\Exception $e){
            // Something was wrong with the data.
            $message = substr($e->getMessage(),0,255);

            $rechargeValues = json_decode($params[8]['value'], true);

            foreach ($rechargeValues as $family){
                $familyName = $this->util->getRechargeFamilyName($family['CDFAMILISALD']);

                $DSNOTIFICAAPP = "Ocorreu um erro ao realizar uma recarga na família " . $familyName['NMFAMILISALD'] . ". Descrição: " . $message;
                $NRNOTIFICAAPP = $this->util->newCode('NOTIFICACAOAPP', 10);
                $CDCLIENTENOTIFAPP = $consumer['CDCLIENTE'];
                $CDCONSUMINOTIFAPP = $family['CDCONSUMIDOR'];
                $CDTIPONOTIFICAAPP = '003';

                //$this->util->saveNotification($NRNOTIFICAAPP, $CDCLIENTENOTIFAPP, $CDCONSUMINOTIFAPP, $CDTIPONOTIFICAAPP, $DSNOTIFICAAPP);
            }

            $response->addMessage(new Message($message));
            $result = false; // Indicates something went wrong.
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RechargeCardRepository', array('RESULT' => $result)));
        }
    }

    //Check if consumer is already registered in TRANSACAOCONS
    public function consumerAlreadyRegistered($consumer, $consumersRegistered){
        foreach ($consumersRegistered as $cons) {
            if($consumer == $cons){
                return true;
            }
        }
        return false;
    }

    public function getSitefPorFilial(Request\Filter $request, Response $response) {
        try {
            $params     = $request->getFilterCriteria()->getConditions();
            $CDFILIAL   = $params[0]['value'];

            $paramsSitef = $this->rechargeService->getSitefPorFilial($CDFILIAL);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetSitefPorFilialRepo', $paramsSitef));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function getEsitefGeral() {
        $esitefDetails = $this->rechargeService->getEsitefDetails();
        if (empty($esitefDetails)) {
            $esitefDetails = array(array('array vazio'));
        } else {
            if ((empty($esitefDetails[0]['IDLOJAESITEF'])) || (empty($esitefDetails[0]['CDLOJAESITEF']))) {
                $esitefDetails = array(array('array vazio'));
            }
        }
        return $esitefDetails;
    }

    public function getPaymentTerms(Request\Filter $request, Response $response){
        $text = "TERMOS DE USO

        1. INTRODUÇÃO
        
        Bem vindo à Appetite Gourmet!
        
        Prezado Responsável, o App Appetite Gourmet é um aplicativo de acompanhamento de consumos alimentícios escolar. Os serviços de alimentação serão prestados pela MARIA DAS GRACAS DANTAS AZEVEDO CANTINAS - ME, pessoa jurídica inscrita no CNPJ sob nº 24.687.382/0001-31, com sede no município de Recife/PE, pela NASCIMENTO E DANTAS SERVICOS DE ALIMENTACAO LTDA - ME, pessoa jurídica inscrita no CNPJ sob nº CNPJ_28.319.352/0001-23, com sede no município de Caruaru/PE e pela RGM SERVICOS DE ALIMENTACAO LTDA, pessoa jurídica inscrita no CNPJ sob nº 97.534.191/0001-03, com sede no município de Recife/PE.
        
        O aplicativo App Appetite Gourmet é destinado aos pais ou responsáveis (\"Responsável Financeiro\") por alunos (\"Consumidores Dependentes\") matriculados em colégios, cujas cantinas (\"Estabelecimentos\") utilizam sistemas de gestão de vendas implementados em parceria com a empresa de software Teknisa®. O Dependente fará o consumo de produtos nos Estabelecimentos aprovados utilizando o Crédito Pessoal de seu Cartão Virtual Pré-Pago. Ao efetuar o cadastro do Consumidor Dependente, você, na condição de Responsável Financeiro, está concordando com estes Termos de Uso. Leia-os atentamente.
        
        2. ALTERAÇÕES NOS TERMOS
        
        Ocasionalmente, poderemos, a nosso critério, fazer alterações nos Termos. Seu uso contínuo do serviço depois que as alterações tiverem sido feitas constituirá na aceitação das alterações. Se você não quiser continuar usando o serviço segundo a nova versão dos Termos, poderá encerrar os Termos entrando em contato conosco por meio da Administração do Estabelecimento de sua unidade escolar.
        
        3. CADASTRO DE CONSUMIDOR DEPENDENTE E ADESÃO AOS SERVIÇOS
        
        Para que o aplicativo possa ser utilizado, é necessário realizar o cadastro do Consumidor Dependente e Responsável Financeiro através do preenchimento por escrito da ficha de adesão impressa, disponível no Estabelecimento de sua unidade escolar. Você deve manter essas informações sempre atualizadas, pois enviaremos as comunicações relativas ao serviço no e-mail que você indicar ali.
        
        Ao realizar o vínculo de um Consumidor Dependente à sua conta de Responsável Financeiro, você assume que o aluno em questão é seu dependente e que as informações associadas a ele estão corretas.
        
        4. UTILIZAÇÃO DO APLICATIVO APP APPETITE GOURMET
        
        O aplicativo deve ser utilizado somente de acordo com as regras e instruções de utilização, que estão disponibilizadas neste documento.
        
        4.1 CARGA E RECARGA DO CARTÃO VIRTUAL PRÉ-PAGO
        
        Antes de começar a utilizar o aplicativo App Appetite Gourmet, você deverá primeiro realizar uma recarga do Cartão Virtual Pré-Pago pelas formas previstas no aplicativo ou diretamente no colégio (“Recarga Física”).
        
        Não serão efetuadas cobranças de taxas de serviços para recargas, seja via aplicativo ou recarga física.
        
        4.2 FUNCIONALIDADES ON LINE DO RESPONSÁVEL
        
        O Estabelecimento poderá disponibilizar a você o acesso a funcionalidades on line como recarga online, limites diários, restrição de produtos. Com o login e senha cadastrados, você também poderá conferir o saldo do cartão virtual pré-pago e o extrato financeiro sempre que desejar.
        
        4.3. USO DO CARTÃO VIRTUAL PRÉ-PAGO
        
        Para a utilização do Cartão Virtual Pré-Pago nos Estabelecimentos, é importante a definição de uma senha do cartão e/ou cadastramento biométrico. A senha ou cadastro biométrico poderão ser definidos nos primeiros usos do cartão. Caso necessite redefinir a senha, poderá ser solicitado a um representante do Estabelecimento de sua unidade escolar.
        
        O uso do Cartão Virtual Pré-Pago, do login e de suas senhas são pessoais e não devem ser cedidos para terceiros em hipótese alguma. Além disso, você pode alterar a senha de acesso ao aplicativo App Appetite Gourmet sempre que precisar.
        
        4.5 INATIVIDADE DA CONTA DO CONSUMIDOR DEPENDENTE
        
        No caso de inatividade na conta do Consumidor Dependente, não haverá cobrança de taxa de manutenção de conta.
        
        5. COMERCIALIZAÇÃO DE PRODUTOS NOS ESTABELECIMENTOS
        
        Os Estabelecimentos fornecerão os produtos adquiridos com o Cartão Virtual Pré-Pago ao Consumidor Dependente mediante ao respectivo saldo da conta. Em caso de saldo insuficiente, o serviço poderá ser suspenso até que uma nova carga seja realizada.
        
        Os Estabelecimentos poderão, a seu exclusivo critério e responsabilidade, permitir que o saldo seja utilizado além dos recursos disponíveis. Nesse caso, se o saldo de carga vier a se tornar negativo, você deverá regularizar a pendência diretamente com o Estabelecimento emissor do seu Cartão Virtual Pré-Pago segundo os prazos ou limites ajustados. O Estabelecimento poderá suspender o fornecimento de produtos ao Consumidor Dependente e realizar a cobrança do débito caso o saldo não seja recomposto dentro do prazo concedido.
        
        6. CUSTOS
        
        O uso Cartão Virtual Pré-Pago é isento de taxas de adesão ou manutenção.
        
        7. COMUNICAÇÃO COM OS ESTABELECIMENTOS
        
        Você pode encaminhar perguntas, sugestões, reclamações e solicitações aos estabelecimentos através do e-mail contato@appetitegourmet.com.br.
        
        Os Estabelecimentos poderão entrar em contato sempre que necessário através dos contatos indicados no momento do cadastro do Consumidor Dependente, por isso é importante que você mantenha seus dados sempre atualizados no aplicativo.
        
        8. SUSPENSÃO, INTERRUPÇÃO E MODIFICAÇÃO DOS SERVIÇOS
        
        A Appetite Gourmet, em parceria com a empresa de software Teknisa®, se esforça para que os serviços oferecidos via Internet estejam sempre disponíveis, mas é possível que o sistema enfrente problemas de conectividade ocasionalmente. É importante que você tenha ciência de que o sistema pode, em casos extremos, ficar fora do ar, e que faremos o possível para retomar os serviços o mais rápido possível.
        
        O aplicativo App Appetite Gourmet e quaisquer outros serviços, poderão ser descontinuados a qualquer tempo. Se isso vier a acontecer, os procedimentos relativos ao cancelamento dos serviços serão seguidos.
        
        As aplicações desenvolvidas em parceria com a empresa de software Teknisa® estão em constante atualização. Funcionalidades podem ser modificadas, adicionadas ou retiradas sem prévio aviso.
        
        9. POLÍTICA DE CANCELAMENTO DOS SERVIÇOS
        
        Você poderá cancelar os serviços a qualquer momento, bastando informar a Administração do Estabelecimento de sua unidade escolar.
        
        Em caso de cancelamento por sua iniciativa, você contará com um prazo de até o fim do ano letivo vigente para fazer uso do saldo de carga existente no Cartão ou resgatar, pessoalmente junto ao Estabelecimento de sua unidade escolar, o saldo existente. O resgate é integral caso a compra recente de crédito pessoal tenha sido realizado em dinheiro. Caso tenha a compra recente do crédito pessoal tenha sido realizada em cartão de crédito, débito ou refeição, é cobrada uma taxa de 5% sobre o valor total do resgate.
        
        10. PROTEÇÃO À PRIVACIDADE
        
        A Appetite Gourmet não compartilha as suas informações pessoais de cadastro ou de seus Consumidores Dependentes. Todas as informações de cadastro coletadas são utilizadas para identificação no sistema. O acesso do Responsável Financeiro e do Estabelecimento aos sistemas de gestão de vendas da Teknisa® é protegido por senhas.
        
        SOBRE ESSE TERMO
        
        Podemos modificar estes termos ou quaisquer termos adicionais que sejam aplicáveis a um serviço para, por exemplo, refletir alterações da lei ou mudanças em nossos serviços.";

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('UseTermsRepository', array('text' => $text)));
    }

}