<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Store extends \Zeedhi\Framework\Controller\Simple {

    protected $storeService;
    protected $snackMessage;
    protected $util;

    public function __construct(\Service\StoreService $storeService, \Util\SnackMessage $snackMessage, \Util\Util $util){
        $this->storeService = $storeService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
    }

    public function getStores(Request\Filter $request, Response $response){
        try {
            $stores = $this->storeService->getStores();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('StoreListRepository', $stores));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getWeekSchedule(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDFILIAL = $params[0]['value'];
            $CDLOJA = $params[1]['value'];

            /* Gets the store's week schedules. */
            $schedule = $this->storeService->getWeekSchedule($CDFILIAL, $CDLOJA);

            foreach ($schedule as &$day){
                /* Sets the day's name. */
                $day['NMDIA'] = $this->getNameDayByNumber($day['NRDIAATENDELOJA']);

                /* Formats the times. */
                $hrini = $day['HRINIATENDELOJA'];
                $hrfim = $day['HRFIMATENDELOJA'];
                $hrini = substr($hrini, 0, 2).":".substr($hrini, 2);
                $hrfim = substr($hrfim, 0, 2).":".substr($hrfim, 2);
                $day['HORARIO'] = $hrini." às ".$hrfim;
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetWeekScheduleRepo', $schedule));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getNameDayByNumber($dayNumber){
        $days = array(
            1=> "Domingo",
            2=> "Segunda-Feira",
            3=> "Terça-Feira",
            4=> "Quarta-Feira",
            5=> "Quinta-Feira",
            6=> "Sexta-Feira",
            7=> "Sábado"
        );

        $todayNumber = date("w") + 1;
        return ($todayNumber == $dayNumber) ? "Hoje" : $days[$dayNumber];
    }

    public function getRedirectsStore(Request\Filter $request, Response $response){
        try {
            $RedirectsStore = $this->storeService->getRedirectsStore();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RedirectRepo', $RedirectsStore));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getStoreByBranch(Request\Filter $request, Response $response){
         try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDFILIAL = $params[0]['value'];
            $stores =  $this->storeService->getStoreByBranch($CDFILIAL);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('StorebyBranchRepository', $stores));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }


    public function saveRedirect(Request\Filter $request, Response $response){
         try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = json_decode($params[0]['value'], true);

            $CDFILIAL       = $params['CDFILIAL'];
            $CDLOJA         = $params['CDLOJA'];
            $CDFILDESTINO   = $params['CDFILIAL2'];
            $CDLOJDESTINO   = $params['CDLOJA2'];
            $DTINICIAL      = $params['DTINICIAL'];
            $DTFINAL        = $params['DTFINAL'];

            $CDREDILOJA = $this->util->newCode('REDIRECIONALOJA', 10);

            $this->storeService->saveRedirect($CDREDILOJA, $CDFILIAL, $CDLOJA, $CDFILDESTINO, $CDLOJDESTINO, $DTINICIAL, $DTFINAL);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function redirectDelete(Request\Filter $request, Response $response){
         try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDREDILOJA = $params[0]['value'];

            $this->storeService->redirectDelete($CDREDILOJA);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function changeProductionTime(Request\Filter $request, Response $response){
         try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);

            $CDFILIAL       = $params['CDFILIAL'];
            $CDLOJA         = $params['CDLOJA'];
            $CDOPERADOR     = $params['CDOPERADOR'];
            $NRTEMPPROD     = $params['NRTEMPPROD'];
            $HRTEMPOENTREGA = $params['HRTEMPOENTREGA'];
            $DSOBSERVACAO   = $params['DSOBSERVACAO'];
            $actualTime = $this->storeService->getProductionTime($CDFILIAL, $CDLOJA);

            if($actualTime[0]['NRTEMPPROD'] != "NRTEMPPROD"){
                $DSCAMPOALT = "NRTEMPPROD";
                $this->storeService->logChangeProductionTime($CDFILIAL, $CDLOJA, $DSCAMPOALT, $NRTEMPPROD, $CDOPERADOR, $DSOBSERVACAO);
            }

            if($actualTime[0]['HRTEMPOENTREGA'] != "HRTEMPOENTREGA"){
                $DSCAMPOALT = "HRTEMPOENTREGA";
                $this->storeService->logChangeProductionTime($CDFILIAL, $CDLOJA, $DSCAMPOALT, $HRTEMPOENTREGA, $CDOPERADOR, $DSOBSERVACAO);
            }

            $this->storeService->changeProductionTime($CDFILIAL, $CDLOJA, $NRTEMPPROD, $HRTEMPOENTREGA);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ChangeProductionTimeRepository', array(array('okay' => 'okay'))));

        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getCheckMode(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $checkMode = $this->storeService->getCheckMode();
            if ($checkMode['IDVALIDAATEN'] === 'B' || $checkMode['IDVALIDAATEN'] === 'C')
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CheckModeRepository', $checkMode));
            else
                throw new \Exception($this->snackMessage->getMessage('12'));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getPlaces(Request\Filter $request, Response $response){
		try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDCLIENTE = $params[0]['value'];
			$filiais = $this->storeService->getPlaces($CDCLIENTE);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('PlacesRepository', $filiais));
		} catch(\Exception $e){
			
		}
    }

    public function buildMenu(Request\Filter $request, Response $response){
		try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDCLIENTE = $params[0]['value'];
            $CDFILIAL = $params[1]['value'];
            $NRCONFTELA = $params[2]['value'];
            $CDLOJA = $params[3]['value'];
            $CDCAIXA = $params[4]['value'];
			$menu = $this->storeService->buildMenu($CDCLIENTE, $CDFILIAL, $NRCONFTELA, $CDLOJA, $CDCAIXA);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductsRepository', $menu));
		} catch (\Exception $e) {
			throw $e;
		}
	}

}