<?php
namespace Service;

class CreditCard {

    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function checkDuplicatedCreditCardToken($credtiCardToken) {
        $params = array(
            ':DSTOKEN' => $credtiCardToken
        );
        
        return $this->entityManager->getConnection()->fetchAssoc(\Util\Query::CHECK_DUPLICATED_CREDIT_CARD_TOKEN, $params);
    }

    public function getNrseqbandeira($flag){
        $params = array(
            ':IDESITEF' => $flag
        );
        return $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_NRSEQBANDEIRA, $params);
    }

    public function storeCreditCardToken($CDCARTCRED, $instantBuyKey, $CDCONSUMIDOR, $CDCLIENTE, $holderName, $numbers, $NRSEQBANDEIRA, $finalExpirationDate, $default, $md5){
        $params = array(
            ':CDCARTCRED' => $CDCARTCRED,
            ':DSTOKEN' => $instantBuyKey,
            ':IDTPCARTCRED' => $default,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE,
            ':NMTITULAR' => $holderName,
            ':NRULTNUM' => $numbers,
            ':NRSEQBANDEIRA' => $NRSEQBANDEIRA,
            ':DTEXPIRACAO' => $finalExpirationDate,
            ':CARTCREDMD5' => $md5
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::STORE_CREDIT_CARD_TOKEN, $params);
    }

    public function getCreditCard($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CREDIT_CARD, $params);
    }
    
    public function getUseCvvFlag($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_USE_CVV, $params);
    }

    public function getFilialKeys($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_FILIAL_KEYS, $params);
    }

    public function getAvaliableFlags($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_AVALIABLE_FLAGS, $params);
    }

    public function deleteCreditCard($creditCards) {
        foreach ($creditCards as $key => $creditCard) {
            $params = array(
                ':CDCARTCRED' => $creditCard["CDCARTCRED"]
            );
            $this->entityManager->getConnection()->executeQuery(\Util\Query::DELETE_CREDIT_CARD, $params);
        }
    }

    public function setAllCreditCardsToFlagO($CDCLIENTE, $CDCONSUMIDOR) {
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::SET_ALL_CREDIT_CARDS_TO_FLAG_O, $params);
    }

    public function setCreditCardToFlagP($CDCARTCRED) {
        $params = array(
            ':CDCARTCRED' => $CDCARTCRED
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::SET_CREDIT_CARD_TO_FLAG_P, $params);
    }

    public function getAutoRechargeDetails($CDCLIENTE, $CDCONSUMIDOR, $CDFILIAL){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFILIAL' => $CDFILIAL
        );
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_AUTO_RECHARGE_DETAILS, $params);

        foreach ($result as &$family){
            $family['VRMINSALDORRECAR'] = number_format(floatval($family['VRMINSALDORRECAR']), 2, ',', ' ');
            $family['VRRECARGARAUT'] = number_format(floatval($family['VRRECARGARAUT']), 2, ',', ' ');
        }

        return $result;
    }

    public function confirmAutoRechargeDetails($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRMINSALDORRECAR, $VRRECARGARAUT, $CDFILIAL){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD
        );
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_AUTO_RECHARGE_INFO, $params);

        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':VRMINSALDORRECAR' => $VRMINSALDORRECAR,
            ':VRRECARGARAUT' => $VRRECARGARAUT
        );

        if (empty($result)){
            $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_AUTO_RECHARGE, $params);
        }
        else {
            $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_AUTO_RECHARGE, $params);
        }

        return $this->getAutoRechargeDetails($CDCLIENTE, $CDCONSUMIDOR, $CDFILIAL);
    }

    public function saveAutoRechargeLog($CDCLIENTE, $CDCONSUMIDOR, $DSLOGCREDCONS){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':DSLOGCREDCONS' => $DSLOGCREDCONS
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::SAVE_AUTO_RECHARGE_LOG, $params);
    }

    public function saveNewCardLog($CDCLIENTE, $CDCONSUMIDOR, $DSLOGCREDCONS){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':DSLOGCREDCONS' => $DSLOGCREDCONS
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::SAVE_NEW_CARD_LOG, $params);
    }

}