<?php

namespace Service;

class Login {

    const EMPTY_SPACE_1 = ' ';
    const EMPTY_SPACE_2 = '  ';


    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getClients() {
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CLIENTS);
    }

    public function getClientsWithFilter($NMFANTCLIE) {
        $params = array(
            ':NMFANTCLIE' => $NMFANTCLIE
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CLIENTS_WITH_FILTER, $params);
    }

    public function getConsumerByLoginClientless($NMLOGINCONS){
        $params = array(
            ':NMLOGINCONS' => $NMLOGINCONS
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_BY_LOGIN_CLIENTLESS, $params);
    }

    public function getConsumerByLogin($CDCLIENTE, $NMLOGINCONS, $takeAll = false) {
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':NMLOGINCONS' => $NMLOGINCONS
        );
        if($takeAll){
            $retorno = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_BY_LOGIN, $params);
        }else{
            $retorno = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_CONSUMER_BY_LOGIN, $params);
        }

        return $retorno;
    }

    public function getConsumerByName($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_BY_NAME, $params);
    }

    public function getConsumerByParent($CDCLIENTE, $NRCPFRESPCON, $CDFILIAL) {
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':NRCPFRESPCON' => $NRCPFRESPCON,
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_BY_PARENT, $params);
    }

    public function getConsumerByEmail($CDCLIENTE, $DSEMAILCONS, $takeAll = false) {
            $params = array(
                ':CDCLIENTE' => $CDCLIENTE,
                ':DSEMAILCONS' => $DSEMAILCONS
            );
        if($takeAll){
            $retorno = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_BY_EMAIL, $params);
        }else{
            $retorno = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_CONSUMER_BY_EMAIL, $params);
        }

        return $retorno;
    }

    public function getFinancialVisionValues($CDCLIENTE, $CDCONSUMIDOR) {

        $time = mktime(0, 0, 0, 9, 1, 2016);
        $currentDate = getdate($time);
        $m = $currentDate['mon'];
        $y = $currentDate['year'];

        // PERIOD CREDITS
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':DTINI' => '01/'.$m.'/'.$y,
            ':DTFIM' => date("t", $time).'/'.$m.'/'.$y,
        );
        $creditsResult = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PERIOD_CREDITS, $params);
        $credits = $creditsResult[0]['TOTALCREDITOS'] == NULL ? 0.0 : $creditsResult[0]['TOTALCREDITOS'];

        // PERIOD DEBITS
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':DTINI' => '01/'.$m.'/'.$y,
            ':DTFIM' => date("t", $time).'/'.$m.'/'.$y,
        );
        $debitsResult = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PERIOD_DEBITS, $params);
        $debits = $debitsResult[0]['TOTALCREDITOS'] == NULL ? 0.0 : $debitsResult[0]['TOTALCREDITOS'];

        // CURRENT TOTAL BALANCE
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $familyBalance = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_BALANCE_LOGIN, $params);
        $totalBalance = 0.0;
        foreach ($familyBalance as $balance){
            $totalBalance += floatval($balance['VRSALDCONFAM']);
        }

        $saldoInicial = number_format(($totalBalance + $debits - $credits),2);
        $credito = number_format($credits,2);
        $debito = number_format($debits,2);
        $saldoAtual = number_format($totalBalance,2);

        $numberLengthArray = array(strlen($saldoInicial),strlen($credito),strlen($debito),strlen($saldoAtual));
        asort($numberLengthArray);
        $maxLength = $numberLengthArray[3];

        $emptyPlaceCredito = '';
        $emptyPlaceDebito  = '';
        $emptyPlaceSaldoInicial = '';
        $emptyPlaceSaldoAtual = '';


        if(strlen($saldoInicial) < $maxLength){
            for($i = 0; $i <= ($maxLength - strlen($saldoInicial));$i++){
                $emptyPlaceSaldoInicial .= self::EMPTY_SPACE_1;
            }
        }

        if(strlen($credito) < $maxLength){
            for($i = 0; $i <= ($maxLength - strlen($credito));$i++){
                $emptyPlaceCredito .= self::EMPTY_SPACE_1;
            }
        }
        if(strlen($debito) < $maxLength){
            for($i = 0; $i <= ($maxLength - strlen($debito));$i++){
                $emptyPlaceDebito .= self::EMPTY_SPACE_1;
            }
        }
        if(strlen($saldoAtual) < $maxLength){
            for($i = 0; $i <= ($maxLength - strlen($saldoAtual));$i++){
                $emptyPlaceSaldoAtual .= self::EMPTY_SPACE_1;
            }
        }


        $values = array(
            array(
                'NAME' => 'Saldo Inicial',
                'VALUE' => str_replace(".", ",", $saldoInicial)
            ),
            array(
                'NAME' => 'Crédito',
                'VALUE' => str_replace(".", ",", $credito)
            ),
            array(
                'NAME' => 'Débito',
                'VALUE' => str_replace(".", ",", $debito)
            ),
            array(
                'NAME' => 'Saldo Atual',
                'VALUE' => str_replace(".", ",", $saldoAtual)
            )
        );

        return $values;
    }

    public function changePassword($CDCLIENTE, $NMLOGINCONS, $CDSENHACONSMD5) {
        $params = array(
            ':P_CDCLIENTE' => $CDCLIENTE,
            ':P_NMLOGINCONS' => $NMLOGINCONS,
            ':P_CDSENHACONSMD5' => $CDSENHACONSMD5
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::SET_PASSWORD, $params);
    }

    public function updateHashAlteraSenha($CDCONSUMIDOR, $CDCLIENTE, $DSHASHALTERASENHA) {
        $params = array(
            ':CDCONSUMIDOR'      => $CDCONSUMIDOR,
            ':CDCLIENTE'         => $CDCLIENTE,
            ':DSHASHALTERASENHA' => $DSHASHALTERASENHA
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_HASH_ALTERASENHA, $params);
    }

    public function getHash($CDCONSUMIDOR, $CDCLIENTE) {
        try {

            $params = array(
                ':CDCONSUMIDOR' => $CDCONSUMIDOR,
                ':CDCLIENTE' => $CDCLIENTE
            );

            $retorno = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_HASH_BY_CONSUMIDOR, $params);
            return count($retorno) > 0 ? $retorno[0]['DSHASHALTERASENHA'] : null;
        } catch (\Exception $e) {
            return null;
        }
    }

    public function updatePassword($CDCLIENTE, $CDCONSUMIDOR, $CDSENHACONSMD5) {
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDSENHACONSMD5' => $CDSENHACONSMD5
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_PASSWORD, $params);
    }
    public function updatePasswordConversion($CDCLIENTE, $CDCONSUMIDOR, $CDSENHACONSMD5) {
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDSENHACONSMD5' => $CDSENHACONSMD5
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::SET_CONSUMER_PASSWORDS, $params);
    }
    public function getPasswordConversion() {
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_PASSWORDS);
    }

    public function acceptContract($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::ACCEPT_CONTRACT, $params);
    }

    public function getLayoutConfig($CDCLIENTE, $CDTIPOCONS, $CDFILIAL){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDTIPOCONS' => $CDTIPOCONS,
            ':CDFILIAL' => $CDFILIAL
        );
        $layoutConfigClient = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_LAYOUT_BY_CLIENT, $params);
        $layoutConfigDefault = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_DEFAULT_LAYOUT, $params);

        return $this->filterLayoutConfig($layoutConfigClient, $layoutConfigDefault);
    }

    private function filterLayoutConfig($layoutConfigClient, $layoutConfigDefault){
        $finalLayoutConfig = array();

        if($layoutConfigClient['CONSULTASALDO'] && $layoutConfigClient['CONSULTASALDO'] !== null){
            $finalLayoutConfig['CONSULTASALDO'] = $layoutConfigClient['CONSULTASALDO'];
        } else{
            $finalLayoutConfig['CONSULTASALDO'] = $layoutConfigDefault['CONSULTASALDO'];
        }

        if($layoutConfigClient['CONSULTAEXTRATO'] && $layoutConfigClient['CONSULTAEXTRATO'] !== null){
            $finalLayoutConfig['CONSULTAEXTRATO'] = $layoutConfigClient['CONSULTAEXTRATO'];
        } else{
            $finalLayoutConfig['CONSULTAEXTRATO'] = $layoutConfigDefault['CONSULTAEXTRATO'];
        }

        if($layoutConfigClient['RECARREGACARTAO'] && $layoutConfigClient['RECARREGACARTAO'] !== null){
            $finalLayoutConfig['RECARREGACARTAO'] = $layoutConfigClient['RECARREGACARTAO'];
        } else{
            $finalLayoutConfig['RECARREGACARTAO'] = $layoutConfigDefault['RECARREGACARTAO'];
        }

        if($layoutConfigClient['BLOQUEACARTAO'] && $layoutConfigClient['BLOQUEACARTAO'] !== null){
            $finalLayoutConfig['BLOQUEACARTAO'] = $layoutConfigClient['BLOQUEACARTAO'];
        } else{
            $finalLayoutConfig['BLOQUEACARTAO'] = $layoutConfigDefault['BLOQUEACARTAO'];
        }

        if($layoutConfigClient['RESTRICAOALIMENTAR'] && $layoutConfigClient['RESTRICAOALIMENTAR'] !== null){
            $finalLayoutConfig['RESTRICAOALIMENTAR'] = $layoutConfigClient['RESTRICAOALIMENTAR'];
        } else{
            $finalLayoutConfig['RESTRICAOALIMENTAR'] = $layoutConfigDefault['RESTRICAOALIMENTAR'];
        }

        if($layoutConfigClient['ATUALIZACADASTRO'] && $layoutConfigClient['ATUALIZACADASTRO'] !== null){
            $finalLayoutConfig['ATUALIZACADASTRO'] = $layoutConfigClient['ATUALIZACADASTRO'];
        } else{
            $finalLayoutConfig['ATUALIZACADASTRO'] = $layoutConfigDefault['ATUALIZACADASTRO'];
        }

        if($layoutConfigClient['TROCASENHA'] && $layoutConfigClient['TROCASENHA'] !== null){
            $finalLayoutConfig['TROCASENHA'] = $layoutConfigClient['TROCASENHA'];
        } else{
            $finalLayoutConfig['TROCASENHA'] = $layoutConfigDefault['TROCASENHA'];
        }

        if($layoutConfigClient['VISUALIZACARDAPIO'] && $layoutConfigClient['VISUALIZACARDAPIO'] !== null){
            $finalLayoutConfig['VISUALIZACARDAPIO'] = $layoutConfigClient['VISUALIZACARDAPIO'];
        } else{
            $finalLayoutConfig['VISUALIZACARDAPIO'] = $layoutConfigDefault['VISUALIZACARDAPIO'];
        }

        if($layoutConfigClient['RESTRICAOVALOR'] && $layoutConfigClient['RESTRICAOVALOR'] !== null){
            $finalLayoutConfig['RESTRICAOVALOR'] = $layoutConfigClient['RESTRICAOVALOR'];
        } else{
            $finalLayoutConfig['RESTRICAOVALOR'] = $layoutConfigDefault['RESTRICAOVALOR'];
        }

        if($layoutConfigClient['FALECONOSCO'] && $layoutConfigClient['FALECONOSCO'] !== null){
            $finalLayoutConfig['FALECONOSCO'] = $layoutConfigClient['FALECONOSCO'];
        } else{
            $finalLayoutConfig['FALECONOSCO'] = $layoutConfigDefault['FALECONOSCO'];
        }

        if($layoutConfigClient['RECARGAAUTOCARTAO'] && $layoutConfigClient['RECARGAAUTOCARTAO'] !== null){
            $finalLayoutConfig['RECARGAAUTOCARTAO'] = $layoutConfigClient['RECARGAAUTOCARTAO'];
        } else{
            $finalLayoutConfig['RECARGAAUTOCARTAO'] = $layoutConfigDefault['RECARGAAUTOCARTAO'];
        }

        if($layoutConfigClient['EDITACARTAO'] && $layoutConfigClient['EDITACARTAO'] !== null){
            $finalLayoutConfig['EDITACARTAO'] = $layoutConfigClient['EDITACARTAO'];
        } else{
            $finalLayoutConfig['EDITACARTAO'] = $layoutConfigDefault['EDITACARTAO'];
        }

        if($layoutConfigClient['IDUTILBOLNUTRI'] && $layoutConfigClient['IDUTILBOLNUTRI'] !== null){
            $finalLayoutConfig['IDUTILBOLNUTRI'] = $layoutConfigClient['IDUTILBOLNUTRI'];
        } else{
            $finalLayoutConfig['IDUTILBOLNUTRI'] = $layoutConfigDefault['IDUTILBOLNUTRI'];
        }

        return $finalLayoutConfig;
    }

}