<?php

namespace Service;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Recharge {

    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function getRechargeGrid($CDFILIAL, $NRCPFRESPCON, $CDCLIENTE){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRCPFRESPCON' => $NRCPFRESPCON,
            ':CDCLIENTE' => $CDCLIENTE
        );

        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_RECHARGE_GRID, $params);;
    }

    public function getConsumerFamilies($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_FAMILIES, $params);
    }

    public function getFilialData($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        $retorno = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_FILIAL_DATA, $params);
        return $retorno[0];
    }

    public function getFilialEndereco($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        $retorno = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_FILIAL_ENDE, $params);
        return $retorno[0];
    }

    public function insertPendentCredit($NRPEDCRECONS, $VRPEDCRECONS, $CDCLIENTE, $CDCONSUMIDOR, $NRBOLETO){
        $params = array(
            ':NRPEDCRECONS'   => $NRPEDCRECONS,
            ':VRPEDCRECONS'   => $VRPEDCRECONS,
            ':IDPEDCREEXTR'   => 'N',
            ':IDPEDCRECONT'   => 'N',
            ':CDCLIENTE'      => $CDCLIENTE,
            ':CDCONSUMIDOR'   => $CDCONSUMIDOR,
            ':NRBOLETO'       => $NRBOLETO
        );

        $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_PEDICREDCONS, $params);
    }

    public function insertPendentCreditFamily($NRPEDCRECONS, $CDFAMILISALD, $VRPEDCREFAMI){
        $params = array(
            ':NRPEDCRECONS'  => $NRPEDCRECONS,
            ':CDFAMILISALD'  => $CDFAMILISALD,
            ':VRPEDCREFAMI'  => $VRPEDCREFAMI
        );

        $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_PEDICREDFAMI, $params);
    }

    public function insertExtratoCons($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $DSOPEEXTCONS, $VRMOVEXTCONS, $IDTPMOVEXT, $VRSALDCONEXT, $CDTIPORECE, $CDFILIAL){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $NRSEQMOVEXT = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_NRSEQMOVEXT, $params);

        $params = array(
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':DSOPEEXTCONS' => $DSOPEEXTCONS,
            ':VRMOVEXTCONS' => $VRMOVEXTCONS,
            ':IDTPMOVEXT'   => $IDTPMOVEXT,
            ':NRSEQMOVEXT'  => $NRSEQMOVEXT[0]['NRSEQMOVEXT'],
            ':VRSALDCONEXT' => $VRSALDCONEXT,
            ':CDTIPORECE'   => $CDTIPORECE,
            ':CDFILIAL'     => $CDFILIAL
        );
        $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_EXTRATOCONS, $params);
    }

    public function getCurrentBalance($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD
        );

        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_EXTRATOCONS_VRSALDCONEXT, $params);
        return $result;
    }

    public function getEsitefDetails(){
        $params = array();
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ESITEF_DETAILS, $params);
        return empty($result[0]['CDLOJAESITEF']) ? array() : $result;
    }

    public function loadBandeirasPorFilialEsiTef($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAll(self::GET_ESITEFFLAGS_FILIAL, $params);
    }

    const GET_ESITEFFLAGS_FILIAL = "
        SELECT NMBANDEIRA, IDESITEF, CDTIPORECE
          FROM BANDESITEFFIL
         WHERE CDFILIAL = :CDFILIAL
    ";

    public function buscaBandeirasEsiTef(){
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ESITEFFLAGS);
    }

    public function getConsumerDetails($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $retorno = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_DETAILS, $params);
        return $retorno[0];
    }

    public function evolveBalance($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $DTMOVEXTCONS = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_DTMOVEXTCONS, $params);
        $DTMOVEXTCONS = $DTMOVEXTCONS[0]['DTMOVEXTCONS'];

        // $connection = $this->entityManager->getConnection();
        // $storedProcedure = new StoredProcedure($connection, 'ATUALIZA_SALDO_CONS_FKB');
        // $storedProcedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $CDCLIENTE, Param::PARAM_TYPE_STR, strlen($CDCLIENTE)));
        // $storedProcedure->addParam(new Param('P_CDCONSUMIDOR', Param::PARAM_INPUT, $CDCONSUMIDOR, Param::PARAM_TYPE_STR, strlen($CDCONSUMIDOR)));
        // $storedProcedure->addParam(new Param('P_CDFAMILISALD', Param::PARAM_INPUT, $CDFAMILISALD, Param::PARAM_TYPE_STR, strlen($CDFAMILISALD)));
        // $storedProcedure->addParam(new Param('P_DTINIEVOLUCAO', Param::PARAM_INPUT, $DTMOVEXTCONS, Param::PARAM_TYPE_STR, strlen($DTMOVEXTCONS)));
        // $result = $storedProcedure->execute();
    }

    public function insertTransacao($dataset) {
        $params = array(
            ':NRTRANSACAO' => $dataset['NRTRANSACAO'],
            ':DSTRANSACAO' => $dataset['DSTRANSACAO'],
            ':VRMOVEXTCONS' => $dataset['VRMOVEXTCONS'],
            ':CDCLIENTE' => $dataset['CDCLIENTE'],
            ':CDCONSUMIDOR' => $dataset['CDCONSUMIDOR'],
            ':NRNIT' => null
        );

        $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_TRANSACAO, $params);
    }

    public function updateTransacao($dataset) {
        $params = array(
            ':NRTRANSACAO' => $dataset['NRTRANSACAO'],
            ':DSTRANSACAO' => $dataset['DSTRANSACAO'],
            ':NRNIT' => $dataset['NRNIT']
        );

        $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_TRANSACAO, $params);
    }

    public function insertTransacaoCons($dataset) {
        $params = array(
            ':NRTRANSACAO' => $dataset['NRTRANSACAO'],
            ':CDCLIENTE' => $dataset['CDCLIENTE'],
            ':CDCONSUMIDOR' => $dataset['CDCONSUMIDOR']
        );

        $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_TRANSACAOCONS, $params);
    }

    public function isFirstInsertion($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD
        );

        return $this->entityManager->getConnection()->fetchAll(\Util\Query::IS_FIRST_INSERTION, $params);
    }

    public function getOwnerName($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );

        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_BY_NAME, $params);
    }

    public function checkConsumerDetails($CDCLIENTE, $NRCPFRESPCON){
        try {
            $params = array(
                'CDCLIENTE' => $CDCLIENTE,
                'NRCPFRESPCON' => $NRCPFRESPCON
            );
            $details = $this->entityManager->getConnection()->fetchAll(\Util\Query::CHECK_CONSUMER_DETAILS, $params);

            return empty($details[0]['NMRESPCONS']) || empty($details[0]['DSEMAILCONS']);
        } catch (\Exception $e){
            return true;
        }
    }

    public function getOnlinePaymentInfo($CDFILIAL){
        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $PaymentInfo = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_ONLINE_PAYMENT_INFO_BY_FILIAL, $params);

        if($PaymentInfo['IDLOJAESITEF'] === null || $PaymentInfo['CDLOJAESITEF'] === null){
            $PaymentInfo = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_ONLINE_PAYMENT_INFO, array());
        }

        return $PaymentInfo;
    }

    public function getPaymentPlatform($CDFILIAL){
        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $PaymentPlatform = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_PAYMENT_PLATFORM_BY_FILIAL, $params);

        if($PaymentPlatform['IDPLATAFORMAPAG'] === null){
            $PaymentPlatform = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_PAYMENT_PLATFORM, array());
        }

        return $PaymentPlatform;
    }

}