<?php

namespace Odhen\API\Lib;

class SatHttp {

	protected $entityManager;
	protected $util;

 	public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Util\Util $util) {
		$this->entityManager = $entityManager;
		$this->util = $util;

		$this->URL_SAT = $this->util->getXMLParameter('URL_NODE_SERVER');
	}

	const EVENT_INICIAR_IMPRESSORA = 'initPrinter';
	const EVENT_CONSULTAR_SAT = 'TConsultarSAT';
	const EVENT_CONSULTAR_STATUS_OPERACIONAL = 'TConsultarStatusOperacional';
	const EVENT_ENVIAR_DADOS_VENDA = 'TEnviarDadosVenda';
	const EVENT_CANCELAR_ULTIMA_VENDA = 'TCancelarUltimaVenda';

	const STATUS_CONSULTA_SAT_OK = '08000';
	const STATUS_CONSULTA_STATUS_OK = '10000';
	const STATUS_VENDA_OK = '06000';

	public function iniciarImpressora($CDSAT) {
		$printerName = self::getPrinterName($CDSAT);
		if ($printerName) {
			$numSessao = rand(1, 999999);
			$params = array(
				'id' => $numSessao,
				'event' => self::EVENT_INICIAR_IMPRESSORA,
				'printerName' => $printerName
			);
			$result = $this->util->httpRequest($this->URL_SAT, $params);
		} else {
			$result = array(
				'error' => true,
				'message' => 'Aparelho SAT não parametrizado corretamente.'
			);
		}
	}

	private function getPrinterName($CDSAT) {
		switch ($CDSAT) {
			case '0':
				$printerName = 'DIMEP_SAT';
				break;
			case '1':
				$printerName = 'BEMATECH_SAT';
				break;
			default:
				$printerName = false;
				break;
		}
		return $printerName;
	}

	public function consultarSAT() {
		try {
			$numSessao = rand(1, 999999);
			if (!empty($numSessao)) {
				$params = array(
					'id' => $numSessao,
		            'event' => self::EVENT_CONSULTAR_SAT,
		            'params' => array(
		            	'numSessao' => $numSessao
	            	)
		        );
				$respostaSAT = $this->util->httpRequest($this->URL_SAT, $params);
				if ($respostaSAT['error'] == false) {
					$respostaSAT = explode('|', $respostaSAT['response']['data']['data']['retorno']);
					$statusSAT = array(
						'numeroSessao' => $respostaSAT[0],
						'EEEEE' => $respostaSAT[1],
						'mensagem' => $respostaSAT[2]
					);
					if ($statusSAT['EEEEE'] == self::STATUS_CONSULTA_SAT_OK) {
						$result = array(
							'error' => false
						);
						$result = array_merge($result, $statusSAT);
					} else {
						$result = array(
							'error' => true,
							'message' => 'Erro ao comunicar com o SAT: ' . $statusSAT['mensagem']
						);
						$result = array_merge($result, $statusSAT);
					}
				} else {
					$result = $respostaSAT;
				}
			} else {
				$result = array(
					'error' => true,
					'message' => 'Número da sessão SAT vazio.'
				);
			}
		} catch(\Exception $e) {
			$result = array(
				'error' => true,
				'message' => 'Erro ao consultar o SAT. Mensagem: ' . $e->getMessage()
			);
		}
		return $result;
	}

	public function consultarStatusOperacional($CDATIVASAT) {
		try {
			$numSessao = rand(1, 999999);
			if (!empty($numSessao)) {
				if (!empty($CDATIVASAT)) {
					$params = array(
						'id' => $numSessao,
						'event' => self::EVENT_CONSULTAR_STATUS_OPERACIONAL,
						'params' => array(
			            	'numSessao' => $numSessao,
							'codigoDeAtivacao' => $CDATIVASAT
	            		)
					);
					$respostaSAT = $this->util->httpRequest($this->URL_SAT, $params);

					if ($respostaSAT['error'] == false) {
						$respostaSAT = explode('|', $respostaSAT['response']['data']['data']['retorno']);
						$statusSAT = array(
							'numeroSessao' => $respostaSAT[0],
							'EEEEE' => $respostaSAT[1],
							'mensagem' => $respostaSAT[2]
						);
						if ($statusSAT['EEEEE'] == self::STATUS_CONSULTA_STATUS_OK) {
							// toDo: fazer validações corretamente
							$statusSAT['cod'] = $respostaSAT[3];
							$statusSAT['mensagemSEFAZ'] = $respostaSAT[4];
							$statusSAT['ConteudoRetorno'] = $respostaSAT[5];

							for ($i = 6; $i < sizeof($respostaSAT); $i++) {
								$statusSAT['ConteudoRetorno'] += '|' + $respostaSAT[$i];
							}
							$result = array(
								'error' => false
							);
							$result = array_merge($result, $statusSAT);
						} else {
							$result = array(
								'error' => true,
								'message' => 'Erro ao comunicar com o SAT: ' . $statusSAT['mensagem']
							);
							$result = array_merge($result, $statusSAT);
						}
					} else {
						$result = $respostaSAT;
					}
				} else {
					$result = array(
						'error' => true,
						'message' => 'Código do SAT vazio.'
					);
				}
			} else {
				$result = array(
					'error' => true,
					'message' => 'Número da sessão SAT vazio.'
				);
			}
		} catch(\Exception $e) {
			$result = array(
				'error' => true,
				'message' => 'Erro ao consultar o status operacional do SAT. Mensagem: ' . $e->getMessage()
			);
		}
		return $result;
	}

	public function enviarDadosVenda($CDATIVASAT, $xml) {
		try {
			$numSessao = rand(1, 999999);
			if (!empty($numSessao)) {
				if (!empty($CDATIVASAT)) {
					if (!empty($xml)) {
						$params = array(
							'event' => self::EVENT_ENVIAR_DADOS_VENDA,
							'id' => $numSessao,
							'params' => array(
								'numSessao' => $numSessao,
				            	'codigoDeAtivacao' => $CDATIVASAT,
								'xml' => $xml
	            			)
						);
						$respostaSAT = $this->util->httpRequest($this->URL_SAT, $params);
						if ($respostaSAT['error'] == false) {
							$respostaSAT = explode('|', $respostaSAT['response']['data']['data']['retorno']);
							$statusSAT = array(
								'numeroSessao' => $respostaSAT[0],
								'EEEEE' => $respostaSAT[1],
								'CCCC' => $respostaSAT[2],
								'mensagem' => $respostaSAT[3]
							);
							if ($statusSAT['EEEEE'] == self::STATUS_VENDA_OK) {
								$statusSAT['cod'] = $respostaSAT[4];
								$statusSAT['mensagemSEFAZ'] = $respostaSAT[5];
								$statusSAT['arquivoCFeBase64'] = $respostaSAT[6];
								$statusSAT['timeStamp'] = $respostaSAT[7];
								$statusSAT['NRACESSONFCE'] = $respostaSAT[8];
								$statusSAT['valorTotalCFe'] = $respostaSAT[9];
								$statusSAT['CPFCNPJValue'] = $respostaSAT[10];
								$statusSAT['DSQRCODENFCE'] = $respostaSAT[11];
								$result = array(
									'error' => false
								);
								$result = array_merge($result, $statusSAT);
							} else {
								$result = array(
									'error' => true,
									'message' => 'Erro ao comunicar com o SAT: ' . $statusSAT['mensagem']
								);
								$result = array_merge($result, $statusSAT);
							}
						} else {
							$result = $respostaSAT;
						}
					} else {
						$result = array(
							'error' => true,
							'message' => 'O documento XML enviado para o SAT está vazio.'
						);
					}
				} else {
					$result = array(
						'error' => true,
						'message' => 'Código do SAT vazio.'
					);
				}
			} else {
				$result = array(
					'error' => true,
					'message' => 'Número da sessão SAT vazio.'
				);
			}
		} catch(\Exception $e) {
			$result = array(
				'error' => true,
				'message' => 'Erro ao enviar os dados da venda para o SAT. Mensagem: ' . $e->getMessage()
			);
		}
		return $result;
	}

	public function cancelarVendaSAT($CDATIVASAT, $chaveCFe, $xml) {
		try {
			$numSessao = rand(1, 999999);
			if (!empty($numSessao)) {
				if (!empty($CDATIVASAT)) {
					if (!empty($xml)) {
						$params = array(
							'event' => self::EVENT_CANCELAR_ULTIMA_VENDA,
							'id' => $numSessao,
							'params' => array(
								'numSessao' => $numSessao,
				            	'codigoDeAtivacao' => $CDATIVASAT,
				            	'chaveCFe' => $chaveCFe,
								'xml' => $xml
	            			)
						);
						$respostaSAT = $this->util->httpRequest($this->URL_SAT, $params);
						if ($respostaSAT['error'] == false) {
							$respostaSAT = explode('|', $respostaSAT['response']['data']['data']['retorno']);
							$statusSAT = array(
								'numeroSessao' => $respostaSAT[0],
								'EEEEE' => $respostaSAT[1],
								'CCCC' => $respostaSAT[2],
								'mensagem' => $respostaSAT[3]
							);
							if ($statusSAT['EEEEE'] == self::STATUS_VENDA_OK) {
								$statusSAT['cod'] = $respostaSAT[4];
								$statusSAT['mensagemSEFAZ'] = $respostaSAT[5];
								$statusSAT['arquivoCFeBase64'] = $respostaSAT[6];
								$statusSAT['timeStamp'] = $respostaSAT[7];
								$statusSAT['NRACESSONFCE'] = $respostaSAT[8];
								$statusSAT['valorTotalCFe'] = $respostaSAT[9];
								$statusSAT['CPFCNPJValue'] = $respostaSAT[10];
								$statusSAT['DSQRCODENFCE'] = $respostaSAT[11];
								$result = array(
									'error' => false
								);
								$result = array_merge($result, $statusSAT);
							} else {
								$result = array(
									'error' => true,
									'message' => 'Erro ao comunicar com o SAT: ' . $statusSAT['mensagem']
								);
								$result = array_merge($result, $statusSAT);
							}
						} else {
							$result = $respostaSAT;
						}
					} else {
						$result = array(
							'error' => true,
							'message' => 'O documento XML enviado para o SAT está vazio.'
						);
					}
				} else {
					$result = array(
						'error' => true,
						'message' => 'Código do SAT vazio.'
					);
				}
			} else {
				$result = array(
					'error' => true,
					'message' => 'Número da sessão SAT vazio.'
				);
			}
		} catch(\Exception $e) {
			$result = array(
				'error' => true,
				'message' => 'Erro ao enviar os dados da venda para o SAT. Mensagem: ' . $e->getMessage()
			);
		}
		return $result;
	}	

}