function Searchable(config) {
	var self = this;
	var ALL_CRITERIA_KEY = "_ALL";

	this.checkRowVisibilityFromCriteria = function checkRowVisibilityFromCriteria(row, searchCriteria, config) {
		config = config || {};
		return Object.keys(searchCriteria).every(function (criteriaKey) {
			var isRowVisible;
			var field;
			var value;
			if (criteriaKey === ALL_CRITERIA_KEY) {
				isRowVisible = Object.keys(row).some(function (rowKey) {
					field = getFieldFromRowProperty(rowKey);
					value = getValueOfRowProperty(row, rowKey, field);
					return valueMatchesAllFieldsCondition(field, value, searchCriteria, config.ignoreUnexistentFields);
				});
			} else {
				field = getFieldFromRowProperty(criteriaKey);
				value = getValueOfRowProperty(row, criteriaKey, field);
				isRowVisible = valueMatchesPerFieldCondition(field, value, searchCriteria, config.ignoreUnexistentFields);
			}
			return isRowVisible;
		});
	};

	function getFieldFromRowProperty(rowProperty) {
		return self.widget.fields.filter(function(field) {
			return field.name === rowProperty;
		}).shift();
	}

	function getValueOfRowProperty(row, key, field) {
		var value = row[key];
		if (field && field.translate) {
			value = self.i18n(value);
		}
		return value;
	}

	function valueMatchesAllFieldsCondition(field, value, searchCriteria, ignoreUnexistentFields) {
		return (
			(ignoreUnexistentFields || fieldIsFilterable(field)) &&
			!Util.isUndefinedOrNull(value) &&
			 valueMatchesCondition(searchCriteria[ALL_CRITERIA_KEY], value)
		);
	}

	function valueMatchesPerFieldCondition(field, value, searchCriteria, ignoreUnexistentFields) {
		return (
			(ignoreUnexistentFields || fieldIsFilterable(field)) &&
			!Util.isUndefinedOrNull(value) &&
			valueMatchesCondition(searchCriteria[field.name], value)
		);
	}

	function fieldIsFilterable(field) {
		return (
			!Util.isUndefinedOrNull(field) &&
			field.aclIsVisible &&
			(field.showOnList || field.showOnFilter) &&
			!field.skipSearch
		);
	}

	function valueMatchesCondition(condition, fieldValue) {
		var matchesCondition = false;
		if (isStrictCondition(condition)) {
			matchesCondition = ~fieldValue.toString().indexOf(condition.substring(1, condition.length -1));
		} else {
			condition = condition.latinize().toLowerCase();
			condition = Util.escapeRegExpString(condition);
			var latinized = fieldValue.toString().latinize().toLowerCase();
			var anyOrderMatch = condition.trim(" ").replace(/ +/g, ".*");
			matchesCondition = new RegExp(anyOrderMatch, "g").test(latinized.toLowerCase());
		}
		return matchesCondition;
	}

	function isStrictCondition(condition) {
		return Util.stringStartsWith(condition, "\"") && Util.stringEndsWith(condition, "\"");
	}

}