var ZeedhiController = angular.module('ZeedhiController', []);
ZeedhiController.controller('ZeedhiMain', ZeedhiMain);
ZeedhiMain.$inject = ['$scope', '$timeout', 'restEngine', 'eventHandler', 'eventFactory',
	'eventEngine', 'maskEngine', 'localStorageEngine', 'metaDataFactory', 'renderEngine',
	'wizardEngine', 'templateManager', 'SQLStorageStrategy', '$rootScope', 'dataSourceHolder',
	'PropertyDefiner', 'ScreenService', 'ZhFeatures', 'ZhLogger', 'ZhLocation', 'ZHPromise',
	'MetaDataLoader', 'urlEngine', 'ApplicationContext', 'eventAggregator'
];

function ZeedhiMain($scope, $timeout, restEngine, eventHandler, eventFactory,
	eventEngine, maskEngine, localStorageEngine, metaDataFactory, renderEngine,
	wizardEngine, templateManager, SQLStorageStrategy, $rootScope, dataSourceHolder,
	PropertyDefiner, ScreenService, ZhFeatures, ZhLogger, ZhLocation, ZHPromise,
	MetaDataLoader, urlEngine, ApplicationContext, eventAggregator) {

	window.SQLStorageStrategy = SQLStorageStrategy;
	$rootScope.templateManager = templateManager;

	eventEngine.bindEvents(templateManager, [{
		name: "onUpdate",
		code: function() {
			var phase = $scope.$root.$$phase;
			var deffered = ZHPromise.defer();

			if (!(phase == '$apply' || phase == '$digest') && !$scope.applyInProgressZh) {
				$scope.applyInProgressZh = true;
				$scope.$apply();
				$scope.applyInProgressZh = false;
			}
			// We call "promise when" to wait the next js turn and make possible to execute
			// some callback function after the digest/apply
			ZHPromise.when().then(function() {
				deffered.resolve();
			});
			return deffered.promise;
		},
		id: new Date().getTime()
	}]);

	$rootScope.menuBack = true;
	$scope.lang = window.lang || "en_us";
	$scope.$watch('lang', function(lang) {
		templateManager.lang = lang;
	});

	templateManager.inDeveloperMode = window.inDeveloperMode;
	templateManager.developerWebSocketUrl = window.developerWebSocketUrl;

	// set locale of moment.js library
	moment.locale($scope.lang);
	$.currency.setLang($scope.lang);

	var updateFeatures = function() {
		ZhFeatures.usePostData = templateManager.project.usePostData;
	};

	MetaDataLoader.loadAcl();
	MetaDataLoader.loadData().
	then(function() {
		return MetaDataLoader.loadProjectMetadata();
	}).
	then(function() {
		updateFeatures();
		var lastLanguage = localStorageEngine.getLocalVar('language');
		ScreenService.setLanguage(lastLanguage || templateManager.project.defaultLanguage);
	});

	$scope.groupBy = Util.groupBy;
	$rootScope.back = function() {
		if (templateManager.historyItems.length > 1) {
			templateManager.historyItems.pop();
			var last = templateManager.historyItems.pop();
			$scope.openWindow(last);
			if (templateManager.wizard && templateManager.wizard.stages[last]) {
				templateManager.wizard.currentStage = templateManager.wizard.stages[last];
			}
			templateManager.updateTemplate();
		}
	};
	$rootScope.openWindow = function(containerName, maintainWindowFilter) {
		var event;
		var currentContainer = templateManager.container;
		if (currentContainer && currentContainer.beforeClose) {
			event = currentContainer.beforeClose();
			if (event && event.isDefaultPrevented()) {
				return event;
			}
		}
		ScreenService.closePopup(true);
		var process = function(container) {
			var event;
			// If the container to open isn't accessible (its menu is set as readOnly).
			if (!Util.isFalse(container.isAccessible)) {
				if (container.stages) {
					wizardEngine.startWizard(container);
				} else {
					event = renderEngine.changeContainer(containerName);
				}
				if (!event || !event.isDefaultPrevented()) {
					if (container.addToHistory !== false) {
						$scope.addHistoryItem(containerName);
					}
					if (currentContainer && currentContainer.afterClose) {
						event = currentContainer.afterClose();
					}
					$timeout(function() {
						$scope.resizeWindowElements();
					}, 200);
				}
			}
			return event;
		};
		publishBeforeContainerOpenEvent(containerName, maintainWindowFilter);
		return MetaDataLoader.loadContainer(containerName, maintainWindowFilter || {})
			.then(process);
	};

	function publishBeforeContainerOpenEvent(containerName, maintainWindowFilter) {
		if (!Util.inDevToolsMode()) {
			eventAggregator.publish("beforeContainerOpen", {
				containerName: containerName,
				maintainWindowFilter: maintainWindowFilter
			});
		}
	}

	$rootScope.openLink = function(url, newWindow) {
		var event;
		var currentContainer = templateManager.container;
		if (currentContainer && currentContainer.beforeClose && !newWindow) {
			event = currentContainer.beforeClose();
			if (event && event.isDefaultPrevented()) {
				return event;
			}
		}

		if (newWindow) {
			window.open(url, "_blank");
		} else {
			window.location.href = url;
		}
	};

	$scope.notificationClick = function(notification) {
		var events = notification.events;
		if (events) {
			eventEngine.bindEvents(notification, events);
			notification.click();
		}
	};
	$scope.userDataActionClick = function(action) {
		var events = action.events;
		if (events) {
			eventEngine.bindEvents(action, events);
			action.click();
		}
	};
	$scope.getRowProp = function(propName) {
		return $scope.widget.currentRow ? $scope.widget.currentRow[propName] : '';
	};
	$scope.addHistoryItem = function(containerName) {
		if (templateManager.historyItems) {
			var historyItemIndex = templateManager.historyItems.indexOf(containerName);
			if (!templateManager.project.maintainHistoryItems) {
				if (historyItemIndex > -1) {
					// remove the item from historyItems to prevent duplicates
					templateManager.historyItems.splice(historyItemIndex, 1);
				}

				templateManager.historyItems.push(containerName);
			} else if (templateManager.project.maintainHistoryItems && historyItemIndex === -1) {
				templateManager.historyItems.push(containerName);
			}
		}
	};
	$scope.newRow = function(widget) {
		widget.newRow();
	};
	$scope.formatMoney = function(money) {
		return maskEngine.currencyFormat(money) || "0.00";
	};
	$scope.actionClick = Util.buildDebounceMethod(function(action, widget, row) {
		if (!action.aclReadOnly && !action.readOnly) {
			if (row) {
				widget.currentRow = row; //see event
			}
			$scope.currentWidget = widget;
			templateManager.currentWidget = widget;
			var events = action.events;
			if (events) {
				eventEngine.bindEvents(action, events, function() {
					return widget.dataSource ? widget.dataSource.data : [];
				}, function() {
					return widget.currentRow;
				}, function() {
					return widget.selectedRow;
				});
			}
			action.click();
		}
	}, 300, true);
	$scope.showProjectMenu = function(show, windowName) {
		templateManager.project.showMenu = show;
		$scope.openWindow(windowName);
		templateManager.updateTemplate();
	};
	$scope.next = function() {
		return wizardEngine.next(eventFactory, eventHandler);
	};
	$scope.previous = function() {
		return wizardEngine.previous(eventFactory, eventHandler);
	};
	var hasLogged = true;
	$rootScope.setLocalVar = function(name, value, callback) {
		if (this !== $scope && !hasLogged) {
			hasLogged = true;
			ZhLogger.warning("Do NOT use setLocalVar, use localStorageEngine.setDataSource instead." +
				"\nThis function may be asyncronuous depending on the plataform.");
		}
		return localStorageEngine.setLocalVar(name, value, callback);
	};
	$rootScope.getLocalVar = function(name, value, callback) {
		if (this !== $scope && !hasLogged) {
			hasLogged = true;
			ZhLogger.warning("Do not use getLocalVar, use localStorageEngine.getDataSource instead." +
				"\nThis function may be asyncronuous depending on the platform.");
		}
		return localStorageEngine.getLocalVar(name, value, callback);
	};
	$scope.resizeWindowElements = function() {
		renderEngine.resizeWindowElements();
	};
	$scope.messagesToShow = [];
	$scope.addMessagesToShow = function(message) {
		$scope.messagesToShow.push(message);
	};
	$scope.alert = function(message) {
		window.alert(message);
	};
	$scope.abs = function(number) {
		return Math.abs(number);
	};
	$scope.setUserIdentifier = function() {
		var userId = localStorage.getItem("USER_ID");
		if (!userId) {
			userId = Math.random().toString(36).substr(2);
			localStorage.setItem("USER_ID", userId);
		}
	};
	$scope.setUserIdentifier();
	$scope.startUploadFile = function() {
		if (typeof Android !== "undefined") {
			Android.callFileChooser();
		} else {
			ZhLogger.warning("startUploadFile() should not be used outside an WebView");
		}
	};
	PropertyDefiner.defineProperty($scope, function() {
		var name = this.field.name;
		var widget = this.widget ? this.widget : this.__filter();
		var stringGetter = this.widget ? "widget.currentRow." : "__filter().";
		return name.indexOf(".") === -1 ? (widget.currentRow ? widget.currentRow[name] : "") : this.$eval(stringGetter + name);
	}, function(value) {
		var name = this.field.name;
		var widget = this.widget ? this.widget : this.__filter();
		var stringGetter = this.widget ? "widget.currentRow." : "__filter().";
		if (name.indexOf(".") === -1) {
			widget.currentRow[name] = value;
		} else {
			this.tempVal = value;
			this.$eval(stringGetter + name + " = tempVal");
			delete this.tempVal;
		}
		widget.currentRow.changed = true;
	}, "fieldRow");
	$scope.toggleSideMenu = ScreenService.toggleSideMenu;

	window.showProjectMenu = $scope.showProjectMenu;
	window.openWindow = $scope.openWindow;
	window.next = $scope.next;
	window.__back__ = function() {
		if (templateManager.container && templateManager.container.onBack) {
			templateManager.container.onBack();
		} else {
			ScreenService.goBack();
		}
	};
	window.__uploadedFile__ = function(fileInBase64) {
		if (templateManager.fileToUpload) {
			templateManager.fileToUpload.setValue('data:image/png;base64,' + fileInBase64);
			templateManager.updateTemplate();
		}
		delete templateManager.fileToUpload;
	};
	window.__menuClickHardware__ = function() {
		if (templateManager.container.getActions('never').length > 0) {
			$scope.menuHardware = !$scope.menuHardware;
		}
	};
	window.__changePosition__ = function(position) {
		ZhLocation.changePosition(position);
	};
	window.__changeWatchPosition__ = function(position, watchId) {
		ZhLocation.changeWatchPosition(position, watchId);
	};
	window.isKeyboardOpened = false;
	window.__changeKeyboardState__ = function(isKeyboardOpened) {
		window.isKeyboardOpened = isKeyboardOpened;
	};

	window.setLocalVar = $scope.setLocalVar;
	window.getLocalVar = $scope.getLocalVar;
	window.metaDataFactory = metaDataFactory;
}