describe('ErrorReporting', function () {

    var errorReporting;
    var zhLogger;
    var screenService;
    var templateManager;
    var mockWindow;

    beforeEach(function() {
        zhLogger = {
            onError: function() {}
        };
        spyOn(zhLogger, 'onError');

        screenService = {
            errorNotification: function() {}
        };
        spyOn(screenService, 'errorNotification');

        templateManager = {
            projectPromise: {
                then: function(projectCbk) {
                    var errorReporting = projectCbk();
                    return {
                        then: function(cbk) {
                            cbk(errorReporting);
                        }
                    };
                }
            },
            project: {}
        };

        mockWindow = {};

        errorReporting = new ErrorReporting(zhLogger, screenService, templateManager, mockWindow);
    });

    it('Should register watch on ZhLogger on error', function() {
        expect(zhLogger.onError).toHaveBeenCalledWith(jasmine.any(Function));
    });

    it('Should provide global function ErrorReporting.__error', function() {
        expect(typeof ErrorReporting.__error).toBe('function');
    });

    it('Should update native error handler', function() {
        expect(typeof mockWindow.onerror).toBe('function');
    });

    it('ErrorReporting.__error should show error an error notification if errorReporting is undefined', function() {
        templateManager.project.errorReporting = undefined;
        var error = 'test error';
        ErrorReporting.__error(error);
        expect(screenService.errorNotification).toHaveBeenCalledWith(error);
    });

    it('ErrorReporting.__error should not show error an error notification if errorReporting is true', function() {
        templateManager.project.errorReporting = true;
        var error = 'test error';
        ErrorReporting.__error(error);
        expect(screenService.errorNotification).toHaveBeenCalledWith(error);
    });

    it('ErrorReporting.__error should not show error an error notification if errorReporting is false', function() {
        templateManager.project.errorReporting = false;
        var error = 'test error';
        ErrorReporting.__error(error);
        expect(screenService.errorNotification).not.toHaveBeenCalled();
    });

});