function OrderCheckController(ScreenService, templateManager, CheckOrderService, AccountCart, OrderPaymentsRepository, ConsumerRepository, MakeOrderRepository, StoreListRepository, ZHPromise, LoginRepository, BalanceService, QRCodeRepository, MenuCartRepository, ProductsHistoryRepo, Query, ObservationsRepository, ProductService){

    var self = this;

    this.loadCheckScreen = function(widget, stripe){
        widget.dataSource.data = [];
        widget.reload();
        AccountCart.findAll().then(function(cartData){
            widget.dataSource.data = cartData;
            this.updateObs(cartData);
            var promises = [];
            promises.push(AccountCart.findAll());
            promises.push(OrderPaymentsRepository.findAll());
            ZHPromise.all(promises).then(function(data){
                var cart          = data[0];
                var payments     = data[1];
                var productTotal = 0;
                for (var i in cart){
                    productTotal += cart[i].PRICE * cart[i].qtty;
                }
                LoginRepository.findOne().then(function (consumerData){
                    BalanceService.getTotalBalance(consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR']).then(function (totalBalance){
                        this.setVrSaldo(stripe, totalBalance[0]['Sum']);
                        var vrNvSaldo = totalBalance[0]['Sum'] - productTotal;
                        this.setVlNvSaldo(stripe, vrNvSaldo);
                    }.bind(this));
                }.bind(this));
                this.setVlTotal(stripe, productTotal);
                templateManager.updateTemplate();
            }.bind(this));
        }.bind(this));
    };

    this.updateObs = function(obsData){
        obsData.forEach(function(product){
            var query = Query.build().where('CDPRODUTO').equals(product['CDPRODUTO']);
            ObservationsRepository.find(query).then(function (obs){
                product['TXPRODCOMVEN'] = ProductService.parseObservations(product['CDPRODUTO'], product, obs);
                AccountCart.save(obsData);
                templateManager.updateTemplate();
            });
        });
    };

    this.updateCart = function(widget){
        var cart = widget.dataSource.data;
        var selectedProduct = widget.currentRow;
        cart.forEach(function(product){
            if (product["ID"] == selectedProduct['ID']){
            var query = Query.build().where('CDPRODUTO').equals(product['CDPRODUTO']);
                ObservationsRepository.find(query).then(function (obs){
                    product['TXPRODCOMVEN'] = ProductService.parseObservations(product['CDPRODUTO'], product, obs);
                    AccountCart.remove(Query.build()).then(function (){
                        AccountCart.save(widget.dataSource.data);
                        templateManager.updateTemplate();
                    });
                });
            };
        });
    };

    this.confirm = function (stripe){
        var vlTotal = stripe.getField('vlTotal').label.moneyToFloat();
        var paymentsWidget = stripe.container.widgets[1];
        var payments = [];
        var totalPaid = 0;
        MenuCartRepository.findAll().then(function(cartData){
            if (cartData.length){
                for (var i in paymentsWidget.dataSource.data){
                    if (paymentsWidget.dataSource.data[i].PRICE.moneyToFloat() > 0) {
                        payments.push(
                            {
                                DSBUTTON   : paymentsWidget.dataSource.data[i].DSBUTTON,
                                VALOR      : paymentsWidget.dataSource.data[i].PRICE
                            }
                        );
                        totalPaid += paymentsWidget.dataSource.data[i].PRICE.moneyToFloat();
                    }
                };
                OrderPaymentsRepository.clearAll().then(function (){
                var paymentDetails = {payments};
                    OrderPaymentsRepository.save(paymentDetails).then(function (){
                        ScreenService.openWindow('qrcodeCheck');
                    });
                });
            }  else {
                ScreenService.showMessage('Não há pedidos.');
               }
        });
    };

    this.goBackMenu = function(stripe){
        MenuCartRepository.clearAll().then(function (){
            OrderPaymentsRepository.findAll().then(function (payments){
                if (payments.length > 0){
                    payments = [];
                }
                OrderPaymentsRepository.save(payments).then(function (){
                    ScreenService.openWindow('menu');
                });
            });
        });
    };

    this.prepareOrder = function(widget, stripe){
        AccountCart.findAll().then(function (cart){
            var totalProdutos = 0;
            var today = new Date();
            ProductsHistoryRepo.findAll().then(function(history){
                var widgetPedido = widget.container.widgets[1]
                for (var i in cart){
                    history.push(
                        {
                            NMGRUPO       : cart[i]['NMGRUPO'],
                            DSBUTTON      : cart[i]['DSBUTTON'],
                            LABELPRICE    : cart[i]['LABELPRICE'],
                            TXPRODCOMVEN  : cart[i]['TXPRODCOMVEN'],
                            CHECKDATE     : moment(today).format('DD/MM/YYYY HH:mm:ss')
                        }
                    );
                        totalProdutos += cart[i]['PRICE'] * cart[i].qtty;
                }
                ProductsHistoryRepo.save(history).then(function (SAVED){
                    widget.dataSource.data = cart;
                    widget.reload();
                    stripe.getField('vlTotal').label = "R$ " + totalProdutos.toFixed(2).replace('.',',');
                });
            });
        });
    };

    this.removeFromCart = function (widget){
        var stripe = widget.container.widgets[0];
        var nvVlTotal = parseFloat(stripe.fields[3].label.replace(",", ".")) - widget.currentRow.PRICE;
        ScreenService.confirmMessage(
            'Remover produto ' + widget.currentRow.DSBUTTON + '?',
            'question',
            function () {
                widget.dataSource.data = widget.dataSource.data.filter(function (itemCart) {
                    return itemCart.ID !== widget.currentRow.ID;
                });
            this.setVlTotal(stripe, nvVlTotal);
            LoginRepository.findOne().then(function (consumerData){
                BalanceService.getTotalBalance(consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR']).then(function (totalBalance){
                    var vrNvSaldo = totalBalance[0]['Sum'] - nvVlTotal;
                    this.setVlNvSaldo(stripe, vrNvSaldo);
                }.bind(this));
            }.bind(this));
            this.newSaveCart(widget.dataSource.data);
        }.bind(this));
    };

    this.newSaveCart = function (widgetData) {
        MenuCartRepository.remove(Query.build()).then(function () {
            MenuCartRepository.save(widgetData);
        });
    };

    this.prepareCheckOrder = function(widget, field){
        field.dataSource.data = [];
        var cart = widget.currentRow;
        var products = widget.widgets[0];
        var query = Query.build().where('CDPRODUTO').equals(cart['CDPRODUTO']);
        ObservationsRepository.find(query).then(function (obs){
            field.dataSource.data = obs;
            products.currentRow = cart;
        });
    };

    this.setVrSaldo = function(stripe, value){
        stripe.getField('vrSaldo').label = value.toMoney();
        return value.moneyToFloat();
    };

    this.setVlTotal = function(stripe, value){
        stripe.getField('vlTotal').label = value.toMoney();
        return value.moneyToFloat();
    };

    this.setVlNvSaldo = function(stripe, value){
        stripe.getField('vrNvSaldo').label = value.toMoney();
        return value.moneyToFloat();
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('OrderCheckController', OrderCheckController);
});