function RechargeController(Query, ScreenService, templateManager, RechargeService, ProfileService, LoginRepository, RechargeValueRepository, eventAggregator, BankDataRepository, LoginController, Utilities, RechargeCardRepository, BuscaBandeirasesitefRepo, CreditCardService, UseTermsRepository, LoginService, FilialRepository) {

    var self = this;
    /* Gets the consumer's authorized store families with zero value, and updates the recharge grid. */
    this.getRechargeGrid = function(rechargeGrid){
        var lang = window.lang;
        rechargeGrid.dataSource.data = [];
        RechargeValueRepository.clearAll().then(function(){
            LoginRepository.findOne().then(function (consumerData){
                RechargeService.getRechargeGrid(consumerData['CDFILIAL'], consumerData['NRCPFRESPCON'], consumerData['CDCLIENTE']).then(
                    function (grid){
                        if (grid[0]["V"]){
                            ScreenService.showMessage('Favor completar o seu cadastro antes de efetuar uma recarga.');
                            ScreenService.openWindow('profile');
                        }
                        if(lang == 'en_us'){
                            var valorTotal = {CDCONSUMIDOR: null, CDFAMILISALD: null, NMCONSUMIDOR: "Valor Total",
                                          NMFAMILISALD: "TOTAL VALUE", NRCPFRESPCON: null, PARENT: null,
                                          PRIMARY: null, VRSALDCONFAM: "0,00"};
                        } else if(lang == 'es'){
                            var valorTotal = {CDCONSUMIDOR: null, CDFAMILISALD: null, NMCONSUMIDOR: "Valor Total",
                                          NMFAMILISALD: "VALOR TOTAL", NRCPFRESPCON: null, PARENT: null,
                                          PRIMARY: null, VRSALDCONFAM: "0,00"};
                        } else{
                            var valorTotal = {CDCONSUMIDOR: null, CDFAMILISALD: null, NMCONSUMIDOR: "Valor Total",
                                          NMFAMILISALD: "VALOR TOTAL", NRCPFRESPCON: null, PARENT: null,
                                          PRIMARY: null, VRSALDCONFAM: "0,00"};
                        }
                        
                        grid.push(valorTotal);

                        /*if (document.body.clientWidth < 400){
                            for (var i in grid){
                                if (grid[i].NMFAMILISALD) grid[i].NMFAMILISALD = grid[i].NMFAMILISALD.split(' ')[0];
                            }
                        }*/

                        rechargeGrid.dataSource.data = grid;
                    },
                    function (){
                        ScreenService.showMessage('Não foi possível acessar o servidor. Tente novamente.');
                    }
                ).then(function() {
                    /* Filtra apenas as linhas que se refiram a cartões */
                    var cards = rechargeGrid.dataSource.data.filter(function (row) {
                        return row.NMCONSUMIDOR === null;
                    });
                    if(cards.length == 1) {
                        this.openRechargeValuePopup(cards[0], templateManager.container.getWidget('chargeValue'));
                    }
                }.bind(this));
            }.bind(this));
        }.bind(this));
    };

    /* Opens the popup for the recharge value. */
    this.openRechargeValuePopup = function(row, widget) {
        if (row.NMCONSUMIDOR == null){
            delete widget.dataSource.data;
            widget.newRow();
            /* If a recharge value was already chosen, restores this value to the popup. */
            if (parseFloat(row.VRSALDCONFAM).toFixed(2) > 0){
                widget.setCurrentRow(row);
                widget.moveToFirst();
            }
            else {
                widget.moveToFirst();
            }
            widget.label = row.NMFAMILISALD;
            widget.currentRow.PARENT = row.PARENT;
            ScreenService.openPopup(widget);
        }
    };

    /* Updates the recharge grid with the value. */
    this.updateRechargeGrid = function(value, dataSource, fieldName, widget) {
        fieldParent = widget.currentRow.PARENT;

        if (value == null || isNaN(value.replace(',','.'))) value = 0;
        else value = value.replace(',','.');

        /* Updates the grid with the recharge value entered in the popup. The for is needed to find the right family in the grid. */
        for (var i in dataSource){
            if (dataSource[i].NMFAMILISALD === fieldName && dataSource[i].PARENT === fieldParent){
                var oldVRSALDCONFAM = parseFloat(dataSource[i].VRSALDCONFAM.replace(',','.')); // Previous value.
                dataSource[i].VRSALDCONFAM = parseFloat(value).toFixed(2).replace('.',','); // New value.
                var currentTotal = parseFloat(dataSource[dataSource.length - 1].VRSALDCONFAM.replace(',','.')); // Previous total value.

                // Calculates the new total value (THE RIGHT WAY).
                dataSource[dataSource.length - 1].VRSALDCONFAM = parseFloat(currentTotal - oldVRSALDCONFAM + parseFloat(dataSource[i].VRSALDCONFAM.replace(',','.'))).toFixed(2).replace('.',',');

                templateManager.updateTemplate();
            }
        }
        ScreenService.closePopup();

        templateManager.container.getWidget('rechargeGrid').activate();
    };

    /* Sets the total recharge value for the label at the end of the recharge wizard. */
    this.setRechargeLabel = function(widget, fieldsWidget) {
        fieldsWidget.newRow();
        fieldsWidget.currentRow.$error = {};

        RechargeValueRepository.findAll().then(function (rechargeValues){
            valorTotal = rechargeValues[rechargeValues.length - 1].VRSALDCONFAM;
            widget.getField('vrTotalPedido').label = 'R$ ' + valorTotal;

            var row = fieldsWidget.currentRow;

            row.valorPedido = parseFloat(valorTotal.replace(',', '.')).toFixed(2);
            fieldsWidget.setCurrentRow(row);
            templateManager.updateTemplate();
        });
    };

    this.isValidCardNumber = function (field, row) {
        if (field.field) {
            //É um elemento
            var cardNumber = field.val().replace(/ /g, '').replace(/_/g, '');
        } else {
            //Não é elemento, busca na row
            var cardNumber = row.cardNumber.replace(/ /g, '');
        }

        var valid = true;

        if (cardNumber.length > 1) {
            if (!cardNumber.startsWith(this.visaNumbers) && !cardNumber.startsWith(this.masterNumbers)) {
                valid = false;
            }
        }

        return valid;
    }

    this.setInvalidCard = function (fieldName, widget) {
        this.cleanFieldErrors(fieldName, widget);
        widget.currentRow.$error[fieldName] = [{"message" : "cartão não aceito"}];
    }

    this.cleanFieldErrors = function (fieldName, widget) {
        if (widget.currentRow.$error[fieldName]) {
            delete widget.currentRow.$error[fieldName];
        }
    }

    this.isExpiredCard = function (date) {
        var valid = true;
        var year = date.substr(date.length - 2);
        var month = date.substring(0, 2);

        var today = new Date();
        var currentMonth = ((today.getMonth() + 1).toString()).fixLength(2, "0");
        var currentYear = (today.getFullYear().toString().substr(2)).fixLength(2, "0");

        if (year < currentYear) {
            valid = false;
        } else if (year == currentYear) {
            if (month < currentMonth) {
                valid = false;
            }
        }

        return valid;
    }

    this.isValidCardExpiration = function (date) {
        var month = date.substring(0, 2);

        return month.between(1, 12);
    }

    String.prototype.fixLength = function (length, completeWhiteSpacesWith) {
        var dif = length - this.length;
        var str = "";
        if (dif > 0) {
            for (var i = 0; i < dif; i++) {
                str += completeWhiteSpacesWith;
            }
        }
        str += this;
        return str;
    }

    this.validateCardExpirationField = function (row, widget, fieldName) {
        var cardExpiration = row.cardExpiration || "";
        var date = cardExpiration.replace('/', '');

        if (!this.isValidCardExpiration(date) && date) {
            this.cleanFieldErrors(fieldName, widget);
            widget.currentRow.$error[fieldName] = [{"message" : "data inválida"}];
        } else if (!this.isExpiredCard(date) && date) {
            this.cleanFieldErrors(fieldName, widget);
            widget.currentRow.$error[fieldName] = [{"message" : "data vencida"}];
        }
    }

    String.prototype.between = function (first, second) {
        return this >= first && this <= second;
    };

    String.prototype.startsWith = function (array){
        for (var i = 0; i < array.length; i++) {
            if (this.slice(0, array[i].length) == array[i]) {
                return true;
            }
        }
        return false;
    };

    this.validateField = function (widget, field) {
        this.cleanFieldErrors(field.name, widget);
        if (!widget.currentRow[field.name]  ) {
            widget.currentRow.$error[field.name] = [{"message" : "obrigatório", "class": "field-require"}];
        }
    };

    this.checkEmptyCardField = function (row, widget, field){
    	if (!row.cardNumber) {
    		this.hideCardFlag(row, widget);
    	} else {
    		if (!this.isValidCardNumber(field, row)) {
                if (field.field){
    			    this.setInvalidCard(field.field.name, widget);
                } else {
                    this.setInvalidCard(field.name, widget);
                }
    		}
    	}
    }

    this.setCards = function (creditCardWidget) {
        var cardFlagField = creditCardWidget.getField('cardFlag');
        var cardImageField = creditCardWidget.getField('cardImage');

        BuscaBandeirasesitefRepo.findAll().then(function (bandeiras) {
            if (bandeiras.length == 0){
                FilialRepository.findOne().then(function (filial){
                    LoginService.loadBandeirasEsiTef(filial.CDFILIAL).then(function (bandeiras){
                        if (bandeiras.length > 0){
                            cardFlagField.dataSource.data = bandeiras;
                            cardImageField.source = "images/cards/default.png";
                            creditCardWidget.dataSource.reload();
                        } else{
                            ScreenService.showMessage('Não há bandeiras cadastradas.');
                            ScreenService.openWindow('parentDashboard');
                        }
                    });
                });
            } else {
                cardFlagField.dataSource.data = bandeiras;
                cardImageField.source = "images/cards/default.png";
                creditCardWidget.dataSource.reload();
            }
        });
    };

    this.setCardFlag = function (creditCardWidget) {
        var cardFlagField = creditCardWidget.getField('cardFlag');
        var cardImageField = creditCardWidget.getField('cardImage');

        switch(cardFlagField.value().CDBANDEIRA) {
            case "1":
                cardImageField.source = "images/cards/visa.png";
                break;
            case "2":
                cardImageField.source = "images/cards/mastercard.png";
                break;
            case "3":
                cardImageField.source = "images/cards/american_express.png";
                break;
            case "33":
                cardImageField.source = "images/cards/diners.png";
                break;
            case "41":
                cardImageField.source = "images/cards/elo.png";
                break;
            case "224":
                cardImageField.source = "images/cards/alelo.png";
                break;
            case "225":
                cardImageField.source = "images/cards/alelo.png";
                break;
            case "280":
                cardImageField.source = "images/cards/sodexo.png";
                break;
            case "281":
                cardImageField.source = "images/cards/sodexo.png";
                break;
            default:
                cardImageField.source = "images/cards/default.png";
                break;
        }
    };

    this.handleFields = function(widget){
        cardFlagField = widget.getField('cardFlag');
        cardNumberField = widget.getField('cardNumber');
        cardExpirationField = widget.getField('cardExpiration');
        securityCodeField = widget.getField('securityCode');

        if(cardFlagField.value() !== null){
            cardNumberField.readOnly = false;
            cardExpirationField.readOnly = false;
            securityCodeField.readOnly = false;

            if(cardFlagField.value() == 3){
                cardNumberField.setValue("");
                securityCodeField.setValue("");
                cardExpirationField.setValue("");
                cardNumberField.maxlength = 15;
                securityCodeField.maxlength = 4;
            } else {
                cardNumberField.setValue("");
                securityCodeField.setValue("");
                cardExpirationField.setValue("");
                cardNumberField.maxlength = 16;
                securityCodeField.maxlength = 3;
            }
        } else {
            cardNumberField.setValue("");
            securityCodeField.setValue("");
            cardExpirationField.setValue("");
            
            cardNumberField.readOnly = true;
            cardExpirationField.readOnly = true;
            securityCodeField.readOnly = true;
        }
    };

    this.resetRegisterCardFlag = function(creditCardWidget){
        var cardImageField = creditCardWidget.getField('cardImage');
        cardImageField.source = "images/cards/default.png";
    };

    this.setRegisterCardFlag = function (creditCardWidget){
        var cardNumber = creditCardWidget.getField('cardNumber').value();
         cardNumber = cardNumber.replace(/\s/g, '');
         
        var cardImageField = creditCardWidget.getField('cardImage');

        var cardFlag = "";

        // Visa
        var re = new RegExp("^4");
        if (cardNumber.match(re) != null) cardFlag = "Visa";

        // Mastercard
        re = new RegExp("^5");
        if (cardNumber.match(re) != null) cardFlag = "Mastercard";

        // Amex
        re = new RegExp("^3[47]");
        if (cardNumber.match(re) != null) cardFlag = "Amex";

        // Diners
        re = new RegExp("^36|301|305|38");
        if (cardNumber.match(re) != null) cardFlag = "Diners";

        // Elo
        re = new RegExp("^4011(78|79)|^43(1274|8935)|^45(1416|7393|763(1|2))|^504175|^627780|^63(6297|6368|6369)|(65003[5-9]|65004[0-9]|65005[01])|(65040[5-9]|6504[1-3][0-9])|(65048[5-9]|65049[0-9]|6505[0-2][0-9]|65053[0-8])|(65054[1-9]|6505[5-8][0-9]|65059[0-8])|(65070[0-9]|65071[0-8])|(65072[0-7])|(65090[1-9]|6509[1-6][0-9]|65097[0-8])|(65165[2-9]|6516[67][0-9])|(65500[0-9]|65501[0-9])|(65502[1-9]|6550[34][0-9]|65505[0-8])|^(506699|5067[0-6][0-9]|50677[0-8])|^(509[0-8][0-9]{2}|5099[0-8][0-9]|50999[0-9])|^65003[1-3]|^(65003[5-9]|65004\d|65005[0-1])|^(65040[5-9]|6504[1-3]\d)|^(65048[5-9]|65049\d|6505[0-2]\d|65053[0-8])|^(65054[1-9]|6505[5-8]\d|65059[0-8])|^(65070\d|65071[0-8])|^65072[0-7]|^(65090[1-9]|65091\d|650920)|^(65165[2-9]|6516[6-7]\d)|^(65500\d|65501\d)|^(65502[1-9]|6550[3-4]\d|65505[0-8])");
        if (cardNumber.match(re) != null) cardFlag = "Elo";

        creditCardWidget.currentRow.cardFlag = cardFlag;

        switch(cardFlag){
            case "Visa":
                cardImageField.source = "images/cards/visa.png";
                break;
            case "Mastercard":
                cardImageField.source = "images/cards/mastercard.png";
                break;
            case "Amex":
                cardImageField.source = "images/cards/american_express.png";
                break;
            case "Diners":
                cardImageField.source = "images/cards/diners.png";
                break;
            case "Elo":
                cardImageField.source = "images/cards/elo.png";
                break;
            default:
                cardImageField.source = "images/cards/default.png";
                break;
        }
    };
    
    this.setCardPopupFlagRegister = function (creditCardPopupWidget, creditCardWidget){
        var cardFlag = creditCardPopupWidget.getField('cardFlagPopup').value();
        var cardImageField = creditCardWidget.getField('cardImage');
        var cardImagePopupField = creditCardPopupWidget.getField('cardImagePopup');

        creditCardWidget.currentRow.cardFlag = cardFlag;

        switch(cardFlag){
            case "Visa":
                cardImageField.source = "images/cards/visa.png";
                cardImagePopupField.source = "images/cards/visa.png";
                break;
            case "Mastercard":
                cardImageField.source = "images/cards/mastercard.png";
                cardImagePopupField.source = "images/cards/mastercard.png";
                break;
            case "Amex":
                cardImageField.source = "images/cards/american_express.png";
                cardImagePopupField.source = "images/cards/american_express.png";
                break;
            case "Diners":
                cardImageField.source = "images/cards/diners.png";
                cardImagePopupField.source = "images/cards/diners.png";
                break;
            case "Elo":
                cardImageField.source = "images/cards/elo.png";
                cardImagePopupField.source = "images/cards/elo.png";
                
                break;
            default:
                cardImageField.source = "images/cards/default.png";
                cardImagePopupField.source = "images/cards/default.png";
                break;
        }
        
        templateManager.updateTemplate();
    };

    this.isValidFieldsForSubmit = function (widget) {
        widget.currentRow.$error = {};
        var fields = [
            'cardNumber',
            'cardExpiration',
            'securityCode'
        ];

        for (var i = 0; i < fields.length; i++) {
            if (!widget.currentRow[fields[i]]) {
                widget.currentRow.$error[fields[i]] = [{"message" : "obrigatório", "class": "field-require"}];
            }
        }

        this.validateCardExpirationField(widget.currentRow, widget, widget.getField('cardExpiration').name);

        return Object.keys(widget.currentRow.$error).length === 0;
    }

    this.doBilletRecharge = function(autoBilletRechargeFlag) {
        var row = templateManager.container.widgets[1].currentRow;
        // Check if method is called by selecting a bank or if there's only one bank to choose
        if (autoBilletRechargeFlag) {
            row.selectBank = templateManager.container.widgets[1].currentRow[0].CDBANCO;
        }
        if (row.selectBank === ''){
             ScreenService.showMessage('Selecione uma banco para prosseguir.')
        } else {
            LoginRepository.findOne().then(function (consumerData) {
                ProfileService.getConsumerDetails(consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], consumerData['CDFILIAL']).then(function(consumerDetails) {
                    if (checkBilletRequiredData(consumerDetails[0])) {
                        RechargeValueRepository.findAll().then(function (rechargeValues) {
                            var query = Query.build()
                                            .where('CDBANCO').equals(row.selectBank);
                            BankDataRepository.findOne(query).then(function (bank) {
                                RechargeService.doBilletRecharge(consumerData['CDFILIAL'], consumerData['CDCLIENTE'], row.selectBank, rechargeValues, bank, Utilities.appUrl, Utilities.logoEmail).then(function (billetBack) {
                                    var url = Utilities.billetUrl + billetBack[0].NRPEDCRECONS + '.html';
                                    RechargeValueRepository.clearAll().then(function () {
                                        if (Util.isDesktop()){
                                            var message = 'O boleto foi enviado para seu e-mail. '
                                                        + 'Clique <a href="' + url + '" target=_blank>aqui</a> para abri-lo';
                                        }
                                        else {
                                            var message = 'O boleto foi enviado para seu e-mail.';
                                        }
                                        ScreenService.showMessage(message, "success");
                                        ScreenService.openWindow('parentDashboard');
                                    }.bind(this));
                                }.bind(this));
                            }.bind(this));
                        }.bind(this));
                    }
                }.bind(this));
            }.bind(this));
        }
    };

    this.validateBilletSelection = function() {
        var selectedPayment = templateManager.container.getWidget('choosePayment').currentRow['selectedPayment'];
        if(selectedPayment === 'billet') {
            LoginRepository.findOne().then(function (consumerData) {
                ProfileService.getConsumerDetails(consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], consumerData['CDFILIAL']).then(function(consumerDetails) {
                    checkBilletRequiredData(consumerDetails[0]);
                }.bind(this));
            }.bind(this));
        }
    };

    var checkBilletRequiredData = function(consumerDetails) {
        if (consumerDetails.DSEMAILCONS == null) {
            redirectInvalidField("email");
        } else if (consumerDetails.NMRESPCONS == null) {
            redirectInvalidField("nome do responsável");
        } else if (consumerDetails.NMBAIRRO == null) {
            redirectInvalidField("bairro");
        } else if (consumerDetails.NMMUNICIPIO == null) {
            redirectInvalidField("municipio");
        } else if (consumerDetails.SGESTADO == null) {
            redirectInvalidField("estado");
        }

        return true;
    };

    var redirectInvalidField = function(field) {
        ScreenService.confirmMessage("O campo " + field + " é necessário para prosseguir. Deseja completar seu perfil?", "question",
        function() {
            ScreenService.openWindow("profile");
        },
        function() {
            var paymentWidget = templateManager.container.getWidget('choosePayment');
            var paymentTypes  = paymentWidget.getField('selectedPayment').dataSource.data;
            if(paymentTypes.length > 1) {
                paymentWidget.currentRow['selectedPayment'] = 'creditCard';
            } else {
                paymentWidget.currentRow['selectedPayment'] = '';
            }
        });
        return false;
    };

    this.validateConsumerDetails = function() {
        LoginRepository.findOne().then(function (consumerData) {
            ProfileService.getConsumerDetails(consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], consumerData['CDFILIAL']).then(function(consumerDetails) {
                this.checkConsumerDetails(consumerDetails[0]);
            }.bind(this));
        }.bind(this));
    };

    this.checkConsumerDetails = function(consumerDetails) {

    };

    /*this.openPrintTab = function (HTMLpath) {
        window.open(HTMLpath);
    }*/

    /* Recharge procedure. */
    this.doCardRecharge = function() {
        if (this.isValidFieldsForSubmit(templateManager.container.widgets[1])) {
            var row = templateManager.container.widgets[1].currentRow;

            var valorPedido = row.valorPedido;
            var codigoBandeira = templateManager.container.getWidget('creditCard').getField('cardFlag').value();

            var nrCartaoOrig = row.cardNumber;
            var dtVencimentoOrig = row.cardExpiration;

            var nmCartao = row.cardHolder;
            var nrCartao = row.cardNumber.replace(/ /g, '');
            var dtVencimento = row.cardExpiration.replace('/', '');
            var codSeguranca = row.securityCode;

            LoginRepository.findOne().then(function (consumerData){
                RechargeValueRepository.findAll().then(function (rechargeValues){
                    // Pulling out valorTotal because there's already valorPedido
                    rechargeValues.pop();
                    RechargeService.doCardRecharge(valorPedido, codigoBandeira, dtVencimento, nrCartao, codSeguranca, consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], rechargeValues, consumerData['CDFILIAL'], Utilities.appUrl, Utilities.logoEmail, nmCartao).then(function (paymentResponse){
                        if (paymentResponse[0]['RESULT'] === true){
                            RechargeValueRepository.remove(Query.build()).then(function (){
                                var cardDetails = {
                                    "nrCartao": nrCartaoOrig,
                                    "dtVencimento": dtVencimentoOrig
                                };
                            });
                            ScreenService.openWindow('parentDashboard');
                        }
                    });
                });
            });
        } else{
            ScreenService.showMessage("Favor preencher todos os campos.");
        }
    };

    this.pagSeguroRecharge = function(flag){
        if (this.isValidFieldsForSubmit(templateManager.container.widgets[1])) {
            var row = templateManager.container.widgets[1].currentRow;

            var valorPedido = row.valorPedido;

            var nrCartao = row.cardNumber.replace(/ /g, '');
            var dtVencimento = row.cardExpiration.split('/');
            var mesVencimento = dtVencimento[0];
            var anoVencimento = '20' + dtVencimento[1];
            var codSeguranca = row.securityCode;
            var bin = nrCartao.substring(0, 6);

            var brand = null;

            RechargeService.getSessionId(flag).then(function(session){
                if(session[0]['session_id']){
                    sessionId = session[0]['session_id'];
                } else{
                    sessionId = session;
                }
                try{
                    PagSeguroDirectPayment.setSessionId(sessionId);
                    var cardHash = PagSeguroDirectPayment.getSenderHash();
                    PagSeguroDirectPayment.getBrand({
                        cardBin: bin,
                        success: function(response) {
                            brand = response.brand;
                            var promise = new Promise(function(resolve, reject) {
                                return PagSeguroDirectPayment.createCardToken({
                                    cardNumber: nrCartao,
                                    brand: brand.name,
                                    cvv: codSeguranca,
                                    expirationMonth: mesVencimento,
                                    expirationYear: anoVencimento,
                                    success: function(response) {
                                        resolve(response.card.token);
                                    },
                                    error: function(response) {
                                        reject(response.errors);
                                    },
                                    complete: function(response) {}
                                });
                            })
                            promise.then(function(cardToken){
                                LoginRepository.findOne().then(function (consumerData){
                                    RechargeValueRepository.findAll().then(function (rechargeValues){
                                        rechargeValues.pop();
                                        RechargeService.pagSeguroRecharge(valorPedido, cardHash, cardToken, nrCartao, consumerData.CDCLIENTE, consumerData.CDCONSUMIDOR, rechargeValues, consumerData.CDFILIAL, Utilities.appUrl, Utilities.logoEmail).then(function(paymentResponse){
                                            if (paymentResponse[0]['RESULT'] === true){
                                                ScreenService.showMessage('Pagamento realizado com sucesso!');
                                                ScreenService.openWindow('parentDashboard');
                                            } else{
                                                ScreenService.showMessage(paymentResponse[0]['MESSAGE']);
                                            }
                                        });
                                    });
                                });
                            })
                            .catch(function(error){
                                console.log(error);
                            });
                        },
                        error: function(response) {
                            if(flag){
                                alert("Erro ao pegar bandeira");
                            }else{
                                self.pagSeguroRecharge(true);
                            }
                        },
                        complete: function(response) {}
                    });
                }
                catch(err) {
                    console.log(err.message);
                }
            });
        }
    }

    /* Gets the consumer's details. This function call is identical to the one the Profile Page. */
    this.getConsumerDetails = function(widget) {
        LoginRepository.findOne().then(function (consumerData){
            ProfileService.getConsumerDetails(consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], consumerData['CDFILIAL']).then(function(consumerDetails){
                widget.dataSource.data = consumerDetails;
                widget.moveToFirst();
                templateManager.updateTemplate();
            });
        });
    };

    this.getBankData = function(billetWidget) {
        var selectBankField = billetWidget.getField('selectBank');
        var autoBilletRechargeFlag = true;

        RechargeService.getBankData().then(function(bankData) {
            //for(var i=0;i<4;i++) { bankData.BankDataRepository.pop(); }
            if (bankData.BankDataRepository.length == 1) {
                billetWidget.currentRow = bankData.BankDataRepository;
                this.doBilletRecharge(autoBilletRechargeFlag);
            } else {
                selectBankField.dataSource.data = bankData.BankDataRepository;
            }
        }.bind(this));
    };

    /* Stores the recharge values in a repository to be used later. */
    this.storeRechargeValue = function(){
        var rechargeGrid = templateManager.container.widgets[0].dataSource.data;
        var total = 0;
        rechargeGrid.forEach(function(rechargeValue){
            if(rechargeValue.NMCONSUMIDOR === 'Valor Total'){
                total = rechargeValue.VRSALDCONFAM;
            }
        });
        if(total === '0,00'){
            ScreenService.showMessage("Valor da recarga deve ser maior que R$ 0,00");
        } else{
            var rechargeValues = [];
            /*Grouping up values and clients using PARENT and PRIMARY properties*/
            var rechargeIndex = 0;
            rechargeGrid.forEach(function(object, index, array){
                if(object.VRSALDCONFAM !== null){
                    rechargeGrid.forEach(function(_object, index, array){
                        if(object.PARENT == _object.PRIMARY){
                            if(object.VRSALDCONFAM != "0,00"){
                                valueConsumer = {CDCONSUMIDOR: _object.CDCONSUMIDOR , CDFAMILISALD: object.CDFAMILISALD, NMCONSUMIDOR: _object.NMCONSUMIDOR, VRSALDCONFAM: object.VRSALDCONFAM};
                                rechargeValues.push(valueConsumer);
                            }
                        }
                    });
                }
            });
            //----
            RechargeValueRepository.save(rechargeValues).then(function(){
                var client = Utilities.getClientApp();
                if(client === 'appetite'){
                    return ScreenService.openWindow('paymentTerms');
                } else{
                    return ScreenService.openWindow('token');
                }
            });
        }
    };

    this.setRechargeLabelOnTokenScreen = function(widget) {
		RechargeValueRepository.findAll().then(function (rechargeValues){
            var total = 0;
            var i = 0;
			for (i; i < rechargeValues.length; i++){
                if(rechargeValues[i]['NMCONSUMIDOR'] != 'Valor Total'){
                    total += parseFloat(rechargeValues[i].VRSALDCONFAM.replace(',','.'));
                }
			}
            widget.getField('vrTotalPedido').label = 'R$ ' + total.toFixed(2).replace('.',',');
			widget.currentRow.valorPedido = total;
			templateManager.updateTemplate();
		});
    };

    this.getRechargeFlags = function(widget){
        LoginRepository.findOne().then(function (consumerData){
            CreditCardService.getAvaliableFlags(consumerData['CDFILIAL']).then(function(flags){
                widget.getField('cardFlag').dataSource.data = flags['dataset']['FlagsId'];
            });
        });
    }

    this.instantBuyRecharge = function(){
        var card = templateManager.container.getWidget('TOKEN_LIST_W').currentRow;
        if(!card.cards) {
            return ScreenService.showMessage('Gentileza selecionar um cartão para pagamento.');
        }
        /*if(CreditCardService.getUseCvv()){
            templateManager.container.getWidget('securityCode').currentRow = [];
            ScreenService.openPopup(templateManager.container.getWidget('securityCode'));
        } else{
            card.cards.CVV = null;*/
            ScreenService.confirmMessage(
                "Deseja realizar a recarga com o cartão selecionado?",
                "QUESTION",
                function(){
                    CreditCardService.setCreditCardInformation(card.cards);
                    self.doTokenRecharge();
                },
                function(){}
            );   
        //}
    }
    
    this.instantBuyRechargeWithCVV = function(){
        var card = templateManager.container.getWidget('TOKEN_LIST_W').currentRow;
        if(!card.cards) {
            return ScreenService.showMessage('Gentileza selecionar um cartão para pagamento.');
        }
        card.cards.CVV = templateManager.container.getWidget('securityCode').currentRow.CVV;
        ScreenService.closePopup();
        ScreenService.confirmMessage(
            "Deseja realizar a recarga com o cartão selecionado?",
            "QUESTION",
            function(){
                CreditCardService.setCreditCardInformation(card.cards);
                self.doTokenRecharge(true);
            },
            function(){}
        );
    }

    this.doTokenRecharge = function(isCvvTransaction = false) {
        var codigoPedido = '01921902919210';
        var row = templateManager.container.widgets[0].currentRow;
        var valorPedido = row.valorPedido;
        LoginRepository.findOne().then(function (consumerData){
            RechargeValueRepository.findAll().then(function (rechargeValues){
                CreditCardService.doTokenRecharge(valorPedido, codigoPedido, rechargeValues, consumerData['CDFILIAL'], CreditCardService.getCreditCardInformation(), Utilities.appUrl, Utilities.logoEmail)
                .then(function (paymentResponse){
                    if(!!paymentResponse.dataset.TokenTransaction.error) {
                        if(!!isCvvTransaction){
                            ScreenService.showMessage(paymentResponse.dataset.TokenTransaction.message, 'ERROR');   
                        } else{
                            templateManager.container.getWidget('securityCode').currentRow = [];
                            ScreenService.openPopup(templateManager.container.getWidget('securityCode'));   
                        }
                    } else {
                        ScreenService.showMessage(paymentResponse.dataset.TokenTransaction.message, 'SUCCESS');
                        ScreenService.openWindow('parentDashboard');
                    }
                });
            });
        });
    };

    this.checkTermsAgree = function(widget){
        if (widget.getField("agree").checked) {
            ScreenService.openWindow('token');
        } else {
            ScreenService.showMessage('É preciso concordar com os termos de uso para prosseguir.');
        }
    };

    this.paymentText = function(textWidget){
        delete textWidget.dataSource.data;
        textWidget.newRow();

        RechargeService.getPaymentTerms().then(function (paymentDetails){

            textWidget.setCurrentRow({"terms": paymentDetails[0].text});

            templateManager.updateTemplate(); 

        });
    };
    
    this.resetCheckBox = function(widget){
        widget.getField("agree").checked = false;
        templateManager.updateTemplate(); 
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('RechargeController', RechargeController);
});
